package com.cesgroup.bdc.SelfDefinedGroup.controller;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.cesgroup.bdc.SelfDefinedGroup.entity.SelfDefinedGroup;
import com.cesgroup.bdc.SelfDefinedGroup.entity.SelfDefinedGroupUser;
import com.cesgroup.bdc.SelfDefinedGroup.service.ISelfDefinedGroupService;
import com.cesgroup.bdc.SelfDefinedGroup.service.ISelfDefinedGroupUserService;
import com.cesgroup.bdc.logs.entity.LogType;
import com.cesgroup.bdc.logs.service.ILogsService;
import com.cesgroup.bdc.util.CuiPage;
import com.cesgroup.bdc.util.PageVo;
import com.cesgroup.bdc.util.PrimaryKey;
import com.cesgroup.kingkong.core.security.model.IUser;
import com.cesgroup.kingkong.web.BaseServiceController;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 自定义组 前端控制器
 * </p>
 *
 * @author ChenKai
 * @since 2019-07-08
 */
@Controller
@RequestMapping("/selfDefindeGroup")
public class SelfDefinedGroupController extends BaseServiceController<SelfDefinedGroup, ISelfDefinedGroupService> {
	@Autowired
	private ISelfDefinedGroupUserService selfDefinedGroupUserService;

	@Autowired
	private ILogsService logsService;

	@RequestMapping("/index")
	public ModelAndView index() {
		ModelAndView mv = new ModelAndView("selfDefindeGroup/index");
		return mv;
	}

	/**
	 * @param pager
	 * @param entity
	 * @return com.cesgroup.bdc.util.PageVo
	 * @Author ChenKai
	 * @Description 查询自定义列表
	 * @Date 2019-07-08 17:01
	 */
	@RequestMapping("/search")
	@ResponseBody
	public PageVo search(CuiPage<SelfDefinedGroup> pager, SelfDefinedGroup entity,HttpSession session) {
		entity.setDeptId(getOrg(session).getId());
		//entity.setCreateUserId(getUser().getId());
		QueryWrapper<SelfDefinedGroup> wrapper = Wrappers.query(entity);
		IPage<SelfDefinedGroup> p = getBaseService().page(pager, wrapper);
		PageVo<SelfDefinedGroup> pv = PageVo.convertFromMybatisplusPage(p);
		return pv;
	}

	/**
	 * @return com.cesgroup.bdc.util.PageVo
	 * @Author ChenKai
	 * @Description TODO
	 * @Date 2019-07-08 17:01
	 */
	@RequestMapping("/removeGroup")
	@ResponseBody
	public boolean removeDrafts(String ids, HttpServletRequest request) {
		IUser uu = getUser();
		try {
			if (StringUtils.isNotBlank(ids)) {
				String[] idsArr = ids.split(",");

				for (String id : idsArr
					) {
					String title = getBaseService().getById(id).getGroupName();
					logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.DELETE, "成功", "自定义组", "", "自定义组", title);

					selfDefinedGroupUserService.deleteByGroupID(id);
				}
				getBaseService().removeByIds(idsArr);
				return true;
			} else {
				throw new RuntimeException("ids不能为空！");
			}

		} catch (Exception e) {
			throw new RuntimeException("删除失败！");
		}
	}

	@RequestMapping("/loadGroupForm")
	@ResponseBody
	public Map<String, Object> loadGroupForm(String id) {
		Map<String, Object> retMap = new HashMap<>();
		SelfDefinedGroup selfDefinedGroup = getBaseService().getBaseMapper().selectById(id);
		QueryWrapper<SelfDefinedGroupUser> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("GROUP_ID", id);
		List<SelfDefinedGroupUser> list = selfDefinedGroupUserService.getBaseMapper().selectList(queryWrapper);
		retMap.put("id", selfDefinedGroup.getId());
		retMap.put("groupName", selfDefinedGroup.getGroupName());
		retMap.put("describtion", selfDefinedGroup.getDescribtion());
		retMap.put("groupUser", list);
		return retMap;
	}

	@GetMapping("/add")
	public ModelAndView add() {
		ModelAndView mv = new ModelAndView("selfDefindeGroup/add");
		return mv;
	}

	@RequestMapping("/addOrUpdate")
	public ModelAndView addOrUpdate(@RequestParam(value = "id", required = false) String id) {
		ModelAndView mv = new ModelAndView("selfDefindeGroup/addOrUpdate");
		mv.addObject("buskId", id);
		return mv;
	}

	/**
	 * @param selfDefinedGroup:自定义组实体类
	 * @param groupDepts:自定义组关联部门
	 * @return boolean
	 * @Author ChenKai
	 * @Description 新增或者修改自定义组
	 * @Date 2019-07-16 16:54
	 */
	@RequestMapping("/addOrUpdateGroup")
	@ResponseBody
	public Map<String, String> addOrUpdateGroup(SelfDefinedGroup selfDefinedGroup, String groupDepts, HttpServletRequest request, HttpSession session) {
		IUser uu = getUser();
		Map<String, String> retMap = new HashMap<>();
		boolean insertFlag = false;
		boolean updateFlag = false;
		JSONArray jsonArray = new JSONArray(groupDepts);
		int depNum = jsonArray.size();
		int successDepNum = 0;
		if (StringUtils.isBlank(selfDefinedGroup.getId())) {//新增
			selfDefinedGroup.setCreateUserId(uu.getId());
			selfDefinedGroup.setCreateUserName(uu.getUserName());
			selfDefinedGroup.setDeptId(getOrg(session).getId());//创建部门ID
			selfDefinedGroup.setDeptName(getOrg(session).getName());//创建部门名称
			selfDefinedGroup.setId(PrimaryKey.generateId());
			int j = getBaseService().getBaseMapper().insert(selfDefinedGroup);
			if (j > 0) {
				logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.ADD, "成功", "自定义组", "", "自定义组", selfDefinedGroup.getGroupName());
				insertFlag = true;
			}
		} else {//修改
			String deptId = selfDefinedGroup.getId();
			UpdateWrapper<SelfDefinedGroup> updateWrapper = new UpdateWrapper<>();
			updateWrapper.eq("ID", deptId);
			int up = getBaseService().getBaseMapper().update(selfDefinedGroup, updateWrapper);
			selfDefinedGroupUserService.deleteByGroupID(deptId);
			if (up > 0) {
				logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.MODIFY, "成功", "自定义组", "", "自定义组", selfDefinedGroup.getGroupName());
				updateFlag = true;
			}
		}
		for (int i = 0; i < jsonArray.size(); i++) {
			JSONObject deptJson = (JSONObject) jsonArray.get(i);
			SelfDefinedGroupUser selfGroupUser = new SelfDefinedGroupUser();
			selfGroupUser.setId(PrimaryKey.generateId());
			selfGroupUser.setGroupId(selfDefinedGroup.getId());
			selfGroupUser.setDeptId(deptJson.getStr("deptId"));
			selfGroupUser.setDeptName(deptJson.getStr("deptName"));
			int depSuccess = selfDefinedGroupUserService.getBaseMapper().insert(selfGroupUser);
			if (depSuccess > 0) {
				successDepNum++;
			}
		}

		if (insertFlag && successDepNum == depNum) {//新增成功
			retMap.put("insertSuccess", "true");
		} else if (updateFlag && successDepNum == depNum) {//更新成功
			retMap.put("updateSuccess", "true");
		}
		return retMap;
	}

}
