package com.cesgroup.bdc.attachment.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cesgroup.bdc.attachment.dto.FileUploaderResponseDto;
import com.cesgroup.bdc.attachment.entity.Attachment;
import com.cesgroup.bdc.attachment.entity.DiskFile;
import com.cesgroup.bdc.attachment.entity.PrintNum;
import com.cesgroup.bdc.attachment.entity.RePrintApply;
import com.cesgroup.bdc.attachment.service.IAttachmentService;
import com.cesgroup.bdc.attachment.service.IDiskFileService;
import com.cesgroup.bdc.attachment.service.IPrintNumService;
import com.cesgroup.bdc.attachment.service.IRePrintApplyService;
import com.cesgroup.bdc.logs.entity.LogType;
import com.cesgroup.bdc.logs.service.ILogsService;
import com.cesgroup.bdc.message.entity.MsgType;
import com.cesgroup.bdc.message.service.INewMessagesService;
import com.cesgroup.bdc.print.service.IPrintRecordService;
import com.cesgroup.bdc.util.MyFileUtil;
import com.cesgroup.bdc.util.PrimaryKey;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.model.IUser;
import com.cesgroup.kingkong.core.security.service.AuthsystemService;
import com.cesgroup.kingkong.web.BaseServiceController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 附件打印次数表 前端控制器
 * </p>
 *
 * @author ChenKai
 * @since 2019-07-17
 */
@Controller
@RequestMapping("/print-num")
public class PrintNumController extends BaseServiceController<PrintNum, IPrintNumService> {

  /**
   * 默认文件存储路径
   */
  @Value("${file.upload.path}")
  private String fileBasePath;

  @Autowired
  private AuthsystemService authsystemService;

  @Autowired
  private IDiskFileService diskFileService;

  @Autowired
  private IAttachmentService attachmentService;

  @Autowired
  private IRePrintApplyService rePrintApplyService;

  @Autowired
  private INewMessagesService messagesService;

  @Autowired
  private IPrintRecordService printRecordService;

  @Autowired
  private ILogsService logsService;

  /**
   * @return org.springframework.web.servlet.ModelAndView
   * @Author ChenKai
   * @Description 重打印申请跳转
   * @Date 2019-07-17 15:59
   */
  @RequestMapping("/printAgainApply")
  public ModelAndView printAgainApply(@RequestParam("receiveId") String receiveId, @RequestParam("attachId") String attachId) {
    ModelAndView mv = new ModelAndView("mailReceive/printApply");
    mv.addObject("receiveId", receiveId);
    mv.addObject("attachId", attachId);
    return mv;
  }

  /**
   * @param attachmentId:附件id
   * @param businessId:收件id
   * @return boolean
   * @Author ChenKai
   * @Description 点击附件后打印次数减少
   * @Date 2019-07-17 15:58
   */
  @RequestMapping("/updatePrintNum")
  @ResponseBody
  public boolean updatePrintNum(@RequestParam("attachmentId") String attachmentId,
                                @RequestParam("businessId") String businessId,
                                @RequestParam("printTime") String printTime,
                                HttpSession session,
                                HttpServletRequest request) {
    boolean flag = false;
    int i = getBaseService().updatePrintNum(attachmentId, businessId, printTime);

    IUser uu = getUser();
    IOrganization org = getOrg(session);
    printRecordService.addOne(uu, org, attachmentId, businessId);

    if (i > 0) {
      flag = true;
      String diskId = attachmentService.getById(attachmentId).getDiskFileId();
      String title = diskFileService.getById(diskId).getOriginalName();
      logsService.operateLog(request, this.getUser().getId(), this.getUser().getUserName(), LogType.PRINT, "成功", "附件", "", "收件夹", title);
    }
    return flag;
  }

  @RequestMapping(value = "findPrintMail")
  @ResponseBody
  public List<FileUploaderResponseDto> findPrintMail(@RequestParam("attachmentId") String attachmentId, @RequestParam("businessId") String businessId) {
    List<Attachment> attachmentList = attachmentService.selectByModuleAndRelevanceId("mail", attachmentId);
    List<FileUploaderResponseDto> list = new ArrayList<>(attachmentList.size());
    for (Attachment attachment : attachmentList) {
      PrintNum printNum = getBaseService().selectByAttachIdAndReceiveId(attachment.getId(), businessId);
      if (printNum == null) {
        printNum = new PrintNum();
        printNum.setId(PrimaryKey.generateId());
        printNum.setAttachId(attachment.getId());
        printNum.setMialReceiveId(businessId);
        printNum.setPrintNumb("10");
        getBaseService().save(printNum);
      }
      DiskFile diskFile = diskFileService.getById(attachment.getDiskFileId());
      FileUploaderResponseDto dto = new FileUploaderResponseDto(attachment, diskFile, printNum);
      list.add(dto);
    }
    return list;
  }

  /**
   * @param reprintAttachId:重打印附件ID
   * @param reprintAttachName:重打印附件名
   * @param request
   * @return boolean
   * @Author ChenKai
   * @Description 提交重打印申请
   * @Date 2019-07-18 10:59
   */
  @RequestMapping("/sendRepPrint")
  @ResponseBody
  public Map<String, Object> sendRepPrint(@RequestParam("reprintAttachId") String reprintAttachId, @RequestParam("reprintAttachName") String reprintAttachName, @RequestParam("receiveId") String receiveId, HttpServletRequest request) {
    Map<String, Object> retMap = new HashMap<>();
    QueryWrapper<RePrintApply> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("RE_ATTACHMENT_ID", reprintAttachId).eq("APPLY_USER_ID", getUser().getId()).eq("STATUS", "0");
    int count = rePrintApplyService.count(queryWrapper);
    if (count <= 0) {
      String rePrintReason = request.getParameter("rePrintReason");
      String rePrintNum = request.getParameter("rePrintNum");
      IUser user = getUser();
      IOrganization org = authsystemService.getOrganizationByUserId(user.getId());
      RePrintApply rePrintApply = new RePrintApply();
      rePrintApply.setId(PrimaryKey.generateId());
      rePrintApply.setApplyUserId(user.getId());
      rePrintApply.setApplyUserName(user.getUserName());
      rePrintApply.setApplyDeptId(org.getId());
      rePrintApply.setReAttachmentId(reprintAttachId);
      rePrintApply.setReAttachmentName(reprintAttachName);
      rePrintApply.setApplyDeptName(org.getName());
      rePrintApply.setCreateTime(new Date());
      rePrintApply.setRePrintReason(rePrintReason);
      rePrintApply.setRePrintNum(rePrintNum);
      rePrintApply.setStatus("0");
      rePrintApply.setMailReceiveId(receiveId);
      Attachment attachment = attachmentService.getById(reprintAttachId);
      //IOrganization sendOrg = authsystemService.getOrganizationByUserId(attachment.getCreatorId());
      rePrintApply.setSendAttachmentDeptId(attachment.getCreateDeptId());
      rePrintApply.setSendAttachmentDeptName(attachment.getCreateDeptName());
      boolean flag = rePrintApplyService.save(rePrintApply);
      retMap.put("success", flag);
      messagesService.addNewMessage(attachment.getCreateDeptId(), this.getUser().getId(), rePrintApply.getId(), MsgType.NEW_REPRINT_MSG);
    } else {
      retMap.put("moreThan1", "1");
    }

    return retMap;
  }

  @RequestMapping("/updatePrintNum2")
  public ModelAndView updatePrintNum2(@RequestParam("attachmentId") String attachmentId,
                                      @RequestParam("businessId") String businessId,
                                      @RequestParam("canDownload") String canDownload
  ) {
    ModelAndView mv = new ModelAndView("mailReceive/printForm");
    mv.addObject("attachmentId", attachmentId);
    Attachment attachment = attachmentService.getById(attachmentId);
    File ofdFile = attachmentService.getOfdFileByAttachment(attachment);
    if (!MyFileUtil.exists(ofdFile)) {
      attachmentService.addNewOfd(attachment);
    }
    mv.addObject("businessId", businessId);
    mv.addObject("canDownload", canDownload);
    return mv;
  }

  @RequestMapping("/initPrintTimes")
  @ResponseBody
  public String initPrintTimes(@RequestParam("attachmentId") String attachmentId,
                               @RequestParam("businessId") String businessId) {
    PrintNum printNum = getBaseService().selectByAttachIdAndReceiveId(attachmentId, businessId);
    return printNum.getPrintNumb();
  }

  @RequestMapping("/updatePrintNum3")
  public ModelAndView updatePrintNum3(@RequestParam("attachId") String attachId) {
    ModelAndView mv = new ModelAndView("mailReceive/printForm2");
    mv.addObject("attachmentId", attachId);
    return mv;
  }
}
