package com.cesgroup.bdc.attachment.service.impl;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cesgroup.bdc.attachment.dto.FileUploaderResponseDto;
import com.cesgroup.bdc.attachment.entity.Attachment;
import com.cesgroup.bdc.attachment.entity.DiskFile;
import com.cesgroup.bdc.attachment.mapper.AttachmentMapper;
import com.cesgroup.bdc.attachment.service.IAttachmentService;
import com.cesgroup.bdc.attachment.service.IDiskFileService;
import com.cesgroup.bdc.cis.service.CisService;
import com.cesgroup.bdc.suwell.service.SuwellService;
import com.cesgroup.bdc.util.FileReadUtil;
import com.cesgroup.bdc.util.PrimaryKey;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.model.IUser;
import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * <p>
 * 附件 服务实现类
 * </p>
 *
 * @author shen.shaohua
 * @since 2019-07-02
 */
@Service
public class AttachmentServiceImpl extends ServiceImpl<AttachmentMapper, Attachment> implements IAttachmentService {
  @Autowired
  private IDiskFileService diskFileService;

  @Autowired
  private SuwellService suwellService;

  @Autowired
  private CisService cisService;

  /**
   * 默认文件存储路径
   */
  @Value("${file.upload.path}")
  private String fileBasePath;

  /**
   * 当前登录用户
   *
   * @return 用户信息
   */
  private IUser getUser() {
    IUser currentUser = (IUser) SecurityUtils.getSubject().getPrincipal();
    return currentUser;
  }

  /*
   * @param files        上传文件
   * @param module
   * @param relevanceId
   * @param relativePath 保存文件的相对路径
   * @return
   * @throws IOException
   */
  /*public List<FileUploaderResponseDto> saveMultiFile(MultipartFile[] files, String module, String relevanceId,
                                                     String relativePath) throws Exception {
    IUser uu = getUser();
    String userId = uu.getId();
    String userName = uu.getUserName();

    //批量存储的文件列表
    //List<Attachment> userFileList = new ArrayList<Attachment>(files.length);
    //返回值内容
    List<FileUploaderResponseDto> responseDtos = new ArrayList<FileUploaderResponseDto>(files.length);
    for (MultipartFile mFile : files) {
      String originalName = mFile.getOriginalFilename();
      Date now = new Date();
      String suffix = null;//后缀名
      int t = originalName.lastIndexOf('.');
      if (t > -1 && t < originalName.length() - 1) {
        suffix = originalName.substring(t + 1);
      }
      String fileName = UUID.randomUUID().toString();
      if (StrUtil.isNotEmpty(suffix)) {
        fileName += "." + suffix;
      }

      DiskFile diskFile = new DiskFile();
      diskFile.setId(PrimaryKey.generateId());
      diskFile.setFileName(fileName);
      diskFile.setFileSize(mFile.getSize());
      diskFile.setContentType(mFile.getContentType());
      diskFile.setOriginalName(originalName);
      diskFile.setExt(suffix);
      diskFile.setCreateTime(now);
      diskFile.setPath(relativePath + diskFile.getFileName()); //存储路径：相对路径 + 存储名称
      diskFile.setCreatorId(userId);
      diskFile.setCreatorName(userName);
      diskFileService.save(diskFile);//保存文件信息

      Attachment attachment = new Attachment();
      attachment.setId(PrimaryKey.generateId());
      attachment.setDiskFileId(diskFile.getId());
      attachment.setCreateTime(now);
      attachment.setCreatorId(userId);
      attachment.setCreatorName(userName);
      attachment.setModule(module);
      attachment.setRelevanceId(relevanceId);
      this.save(attachment);//保存附件关联

      File destFile = new File(fileBasePath + relativePath);
      if (!destFile.exists()) {
        destFile.mkdirs();
      }
      destFile = new File(fileBasePath + relativePath + diskFile.getFileName());
      mFile.transferTo(destFile);//保存到目标位置

      //读取文件内容
      try {
        String text = FileReadUtil.readText(destFile);
        if (StrUtil.isNotBlank(text)) {
          diskFile.setText(text);
          diskFileService.updateById(diskFile);
          diskFileService.alterContextIndex();
        }
      } catch (Exception e) {
        e.printStackTrace();
      }

      //返回值
      FileUploaderResponseDto responseDto = new FileUploaderResponseDto(attachment, diskFile);
      responseDtos.add(responseDto);
    }
    return responseDtos;
  }*/

  /**
   * @param files        上传文件
   * @param module       所属模块
   * @param relevanceId  关联id
   * @param relativePath 保存文件的相对路径
   * @param user         用户
   * @param org          组织
   * @return
   * @throws IOException
   */
  public List<FileUploaderResponseDto> saveMultiFile2(MultipartFile[] files, String module, String relevanceId,
                                                      String relativePath, IUser user, IOrganization org) throws Exception {
    String userId = user.getId();
    String userName = user.getUserName();

    //返回值内容
    List<FileUploaderResponseDto> responseDtos = new ArrayList<FileUploaderResponseDto>(files.length);
    for (MultipartFile mFile : files) {
      String originalName = mFile.getOriginalFilename();
      Date now = new Date();
      String suffix = null;//后缀名
      int t = originalName.lastIndexOf('.');
      if (t > -1 && t < originalName.length() - 1) {
        suffix = originalName.substring(t + 1);
      }
      String fileName = UUID.randomUUID().toString();
      if (StrUtil.isNotEmpty(suffix)) {
        fileName += "." + suffix;
      }
      File destFile = new File(fileBasePath + relativePath);
      if (!destFile.exists()) {
        destFile.mkdirs();
      }
      destFile = new File(fileBasePath + relativePath + fileName);
      mFile.transferTo(destFile);//保存到目标位置

      //文件加密
      cisService.encryptedFile(destFile);

      DiskFile diskFile = new DiskFile();
      diskFile.setId(PrimaryKey.generateId());
      diskFile.setFileName(fileName);
      diskFile.setFileSize(mFile.getSize());
      diskFile.setContentType(mFile.getContentType());
      diskFile.setOriginalName(originalName);
      diskFile.setExt(suffix);
      diskFile.setCreateTime(now);
      diskFile.setPath(relativePath + fileName); //存储路径：相对路径 + 存储名称
      diskFile.setCreatorId(userId);
      diskFile.setCreatorName(userName);
      diskFileService.save(diskFile);//保存文件信息

      Attachment attachment = new Attachment();
      attachment.setId(PrimaryKey.generateId());
      attachment.setDiskFileId(diskFile.getId());
      attachment.setCreateTime(now);
      attachment.setCreatorId(userId);
      attachment.setCreatorName(userName);
      attachment.setCreateDeptId(org.getId());
      attachment.setCreateDeptName(org.getName());
      attachment.setModule(module);
      attachment.setRelevanceId(relevanceId);
      this.save(attachment);//保存附件关联

      //--------转ofd板式文件-----------------
      if (!"ofd".equalsIgnoreCase(suffix)&&!"zip".equalsIgnoreCase(suffix)&&!"ZIP".equalsIgnoreCase(suffix)&&!"RAR".equalsIgnoreCase(suffix)
              &&!"rar".equalsIgnoreCase(suffix)&&!"7z".equalsIgnoreCase(suffix)) {
        final String sourceFilePath = destFile.getAbsolutePath();
        final String ofdFilePath = sourceFilePath.substring(0, sourceFilePath.lastIndexOf('.')) + ".ofd";
        new Thread(() -> {
          try {
            suwellService.officeToOFD(sourceFilePath, ofdFilePath);
          } catch (Exception e) {
            e.printStackTrace();
          }
        }).start();
      }
      //--------转ofd板式文件-----------------

      //读取文件内容
      try {
        String text = FileReadUtil.readText(destFile);
        if (StrUtil.isNotBlank(text)) {
          diskFile.setText(text);
          diskFileService.updateById(diskFile);
          diskFileService.alterContextIndex();
        }
      } catch (Exception e) {
        e.printStackTrace();
      }

      //返回值
      FileUploaderResponseDto responseDto = new FileUploaderResponseDto(attachment, diskFile);
      responseDtos.add(responseDto);
    }
    return responseDtos;
  }

  /**
   * 删除附件
   *
   * @param id 文件关联ID
   * @author: shen.shaohua
   * @since: 2019/7/11 22:34
   */
  public void deleteFile(String id) {
    Attachment attachment = this.getById(id);
    int c = this.countByDiskFileId(attachment.getDiskFileId(), id);
    if (c < 1) { //没有关联到此文件，删除磁盘文件及相应的.ofd文件
      DiskFile diskFile = diskFileService.getById(attachment.getDiskFileId());
      String destPath = fileBasePath + diskFile.getPath();
      String ofdFilePath = destPath.substring(0, destPath.lastIndexOf('.')) + ".ofd";
      FileUtil.del(destPath);
      FileUtil.del(this.getEncryptedPath(destPath));
      FileUtil.del(ofdFilePath);
      FileUtil.del(this.getEncryptedPath(ofdFilePath));
      diskFileService.removeById(diskFile.getId());
    }
    this.removeById(id);
    //getBaseMapper().deleteById(id);
  }

  public int countByDiskFileId(String diskFileId, String attachmentId) {
    QueryWrapper<Attachment> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("disk_file_id", diskFileId).ne("id", attachmentId);
    int c = this.count(queryWrapper);
    return c;
  }

  /**
   * 获得磁盘文件
   *
   * @param attachmentId
   * @return
   * @author: shen.shaohua
   * @since: 2019/9/16 19:01
   */
  public File getFileByAttachmentId(String attachmentId) {
    Attachment attachment = this.getById(attachmentId);
    return this.getFileByAttachment(attachment);
  }

  /**
   * 获得磁盘文件
   *
   * @param attachment
   * @return
   * @author: shen.shaohua
   * @since: 2019/9/16 19:01
   */
  public File getFileByAttachment(Attachment attachment) {
    String diskId = attachment.getDiskFileId();
    DiskFile diskFile = diskFileService.getById(diskId);
    File file = getFileByDiskFile(diskFile);
    this.guessDecrytedFile(file);
    return file;
  }

  public File getFileByDiskFile(DiskFile diskFile) {
    String relativePath = diskFile.getPath();
    if (relativePath.indexOf(File.separatorChar) < 0) {
      if (File.separatorChar == '/') {
        relativePath = relativePath.replace('\\', File.separatorChar);
      } else if (File.separatorChar == '\\') {
        relativePath = relativePath.replace('/', File.separatorChar);
      }
    }
    String path = fileBasePath + relativePath;
    File file = new File(path);
    return file;
  }

  /**
   * 获得.ofd磁盘文件
   *
   * @param attachment
   * @return
   * @author: shen.shaohua
   * @since: 2019/9/16 19:01
   */
  public File getOfdFileByAttachment(Attachment attachment) {
    File sourceFile = getFileByAttachment(attachment);
    String path = sourceFile.getAbsolutePath();
    path = path.substring(0, path.lastIndexOf('.')) + ".ofd";
    File ofdFile = new File(path);
    this.guessDecrytedFile(ofdFile);
    return ofdFile;
  }

  /**
   * 获得.ofd磁盘文件
   *
   * @param diskFile
   * @return
   * @author: shen.shaohua
   * @since: 2019/9/16 19:01
   */
  public File getOfdFileByDiskFile(DiskFile diskFile) {
    File sourceFile = getFileByDiskFile(diskFile);
    String path = sourceFile.getAbsolutePath();
    path = path.substring(0, path.lastIndexOf('.')) + ".ofd";
    File ofdFile = new File(path);
    this.guessDecrytedFile(ofdFile);
    return ofdFile;
  }

  /**
   * 获得附件list
   *
   * @param module      类型
   * @param relevanceId 关联的id
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/9 17:04
   */
  public List<Attachment> selectByModuleAndRelevanceId(String module, String relevanceId) {
    QueryWrapper<Attachment> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("module", module).eq("relevance_id", relevanceId).orderByDesc("create_time");
    List<Attachment> attachmentList = this.list(queryWrapper);
    return attachmentList;
  }

  /**
   * 替换临时关联id
   *
   * @param tempRelevanceId 临时关联id
   * @param relevanceId     关联id
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/9 16:50
   */
  public Integer updateTempRelevanceId(String tempRelevanceId, String relevanceId) {
    Integer c = getBaseMapper().updateTempRelevanceId(tempRelevanceId, relevanceId);
    return c;
  }

  /**
   * 根据原有的关联信息新增
   *
   * @param oldModule
   * @param oldRelevanceId
   * @param newModule
   * @param newRelevanceId
   * @author: shen.shaohua
   * @since: 2019/7/11 22:30
   */
  public int addFromOld(String oldModule, String oldRelevanceId, String newModule, String newRelevanceId,
                        IUser user, IOrganization org) {
    QueryWrapper<Attachment> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("module", oldModule).eq("relevance_id", oldRelevanceId);
    List<Attachment> attachmentList = this.list(queryWrapper);
    int c = attachmentList.size();
    if (c > 0) {
      final Date now = new Date();
      final String userId = user.getId();
      final String userName = user.getUserName();
      final String orgId = org.getId();
      final String orgName = org.getName();
      attachmentList.forEach(attachment -> {
        attachment.setId(PrimaryKey.generateId());
        attachment.setModule(newModule);
        attachment.setRelevanceId(newRelevanceId);
        attachment.setCreateTime(now);
        attachment.setCreatorId(userId);
        attachment.setCreatorName(userName);
        attachment.setCreateDeptId(orgId);
        attachment.setCreateDeptName(orgName);
      });
      this.saveBatch(attachmentList);
    }
    return c;
  }

  /**
   * 删除附件
   *
   * @param module
   * @param relevanceId
   * @author: shen.shaohua
   * @since: 2019/7/12 0:04
   */
  public void deleteByModuleAndRelevanceId(String module, String relevanceId) {
    Map<String, Object> columnMap = new HashMap<>();
    columnMap.put("module", module);
    columnMap.put("relevance_id", relevanceId);
    Collection<Attachment> attachmentList = this.listByMap(columnMap);
    attachmentList.forEach(attachment -> {
      this.deleteFile(attachment.getId());
    });
  }

  public List<String> getRelevanceIds(String module, String keyword) {
    String originalName = keyword;
    originalName = originalName.replace("/", "//").replace("_", "/_").replace("%", "/%");
    originalName = "%" + originalName + "%";
    List<String> list = getBaseMapper().getRelevanceIds(module, originalName, keyword);
    return list;
  }

  @Override
  public void addNewOfd(Attachment attachment) {
    File file = getFileByAttachment(attachment);
    if (file.getName().toLowerCase().endsWith(".ofd")) {
      return;
    }
    String wordPath = file.getAbsolutePath();
    String ofdPath = wordPath.substring(0, wordPath.lastIndexOf('.')) + ".ofd";
    suwellService.officeToOFD(wordPath, ofdPath);
  }

  /**
   * 获取附件文件
   *
   * @param attachment
   * @return
   * @author: shen.shaohua
   * @since: 2019/9/6 17:49
   */
  @Override
  public File getFile(Attachment attachment) {
    if (attachment == null) {
      return null;
    }
    String path = getFilePath(attachment);
    File f = new File(path);
    this.guessDecrytedFile(f);
    return f;
  }
  /**
   * 获取附件文件路径
   *
   * @param attachment
   * @return
   * @author: shen.shaohua
   * @since: 2019/11/20 16:36
   */
  public String getFilePath(Attachment attachment) {
    if (attachment == null) {
      return null;
    }
    DiskFile diskFile = diskFileService.getById(attachment.getDiskFileId());
    String path = diskFile.getPath();
    if (path.indexOf(File.separatorChar) < 0) {
      if (File.separatorChar == '/') {
        path = path.replace('\\', File.separatorChar);
      } else if (File.separatorChar == '\\') {
        path = path.replace('/', File.separatorChar);
      }
    }
    path = fileBasePath + path;
    return path;
  }

  /**
   * 猜测是否jiem 假如明文文件不存在则进行jiem
   * @param plainFile
   * @return
   */
  public boolean guessDecrytedFile(File plainFile){
    boolean success = false;
    if(!plainFile.exists()){
      String plainPath = plainFile.getAbsolutePath();
      String encryptedPath = this.getEncryptedPath(plainPath);
      success = cisService.decryptedFile(new File(encryptedPath));
    }
    return success;
  }

  /**
   * 获取jm文件路径
   * @param plainPath
   * @return
   */
  public String getEncryptedPath(String plainPath){
    File rootPath = new File(fileBasePath);
    String rootAbsoulutePath = rootPath.getAbsolutePath();
    String encryptedPath = plainPath.replace(rootAbsoulutePath,rootAbsoulutePath+"Jm");
    return encryptedPath;
  }

}
