package com.cesgroup.bdc.auth.service;

import cn.hutool.core.util.StrUtil;
import com.cesgroup.authen4.ws.LogHttpClientWS;
import com.cesgroup.authen4.ws.entity.Log;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

/**
 * 系统日志 service
 *
 * @author shen.shaohua
 * @date 2020-12-8 12:17
 */
@Service
public class SystemLogService {
  @Value("${kingkong.authsystem.tenantId}")
  private String TENANT_ID;

  @Value("${kingkong.authsystem.system-id}")
  private String SYSTEM_ID;

  @Value("${kingkong.authsystem.url}")
  private String host;

  private LogHttpClientWS logHttpClientWS;

  protected LogHttpClientWS getLogHttpClientWS() {
    if (logHttpClientWS == null) {
      System.out.println("系统管理平台url===" + host);
      logHttpClientWS = LogHttpClientWS.getLogHttpClientWS(host);
    }
    return logHttpClientWS;
  }

  /**
   * 登录日志
   *
   * @param userId   用户id
   * @param userName 用户名称
   * @param org      组织部门
   * @param message  具体信息
   * @param ip       客户端ip
   * @author shen.shaohua
   * @date 2020-12-8 13:28
   */
  public void addLoginLog(String userId, String userName, IOrganization org, String message, String ip) {
    Log log = new Log();
    log.setTenantId(TENANT_ID);
    log.setSystemId(SYSTEM_ID);
    log.setLogType("4");
    log.setUserId(userId);
    log.setUserName(userName);
    if (org != null) {
      log.setOrganizeId(org.getId());
      log.setOrgName(org.getName());
    }
    log.setOperate("登录");
    log.setModule("用户登录");
    log.setMessage(message);
    log.setStatus(0);
    log.setIp(ip);

    getLogHttpClientWS().saveLog(log);
  }

  /**
   * 操作日志
   *
   * @param userId   用户id
   * @param userName 用户名称
   * @param message  具体信息
   * @param ip       客户端ip
   * @author shen.shaohua
   * @date 2020-12-8 14:53
   */
  public void addOperateLog(String userId, String userName,
                            String organizeId, String orgName,
                            String module, String message, String ip) {
    Log log = new Log();
    log.setTenantId(TENANT_ID);
    log.setSystemId(SYSTEM_ID);
    log.setLogType("4");
    log.setUserId(userId);
    log.setUserName(userName);
    log.setOrganizeId(organizeId);
    log.setOrgName(orgName);
    log.setOperate("操作");
    log.setModule(StrUtil.isEmpty(module) ? "用户操作" : module);
    log.setMessage(message);
    log.setStatus(0);
    log.setIp(ip);

    getLogHttpClientWS().saveLog(log);
  }
}
