package com.cesgroup.bdc.login.web;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.http.HttpUtil;
import com.cesgroup.authen4.ws.entity.User;
import com.cesgroup.bdc.auth.service.AuthService;
import com.cesgroup.kingkong.core.security.model.IUser;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.*;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.net.URLDecoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping
public class LoginController {
  private Log logger = LogFactory.getLog(getClass());

  @Autowired
  private AuthService authService;

  @RequestMapping(value = {"/login.html", "/login"})
  public ModelAndView index() {
    ModelAndView mv = new ModelAndView("login/login");
    return mv;
  }

  /**
   * 使用ca的ukey登录
   *
   * @return
   */
  @RequestMapping(value = {"/loginCA.html", "/loginCA"})
  public ModelAndView loginCA() {
    ModelAndView mv = new ModelAndView("login/loginCA");
    return mv;
  }

  /**
   * SFGL的单点登录
   *
   * @return
   */
  @RequestMapping(value = "/loginSL")
  public String loginSL(String token, HttpServletRequest request) {
    try {
      //解析token，并且单点登录
      String loginName = URLDecoder.decode(token, "UTF-8");
      UsernamePasswordToken upToken = new UsernamePasswordToken(loginName, "SSL");
      Subject subject = SecurityUtils.getSubject();
      subject.login(upToken);
    } catch (Exception e) {
      e.printStackTrace();
    }
    if (SecurityUtils.getSubject().getPrincipal() != null) {

      return "/index";
    }
    return "redirect:/login";
  }

  /**
   * SFGL的单点登录
   *
   * @return
   */
  @RequestMapping(value = "/loginSL1")
  public void loginSL1(String token, HttpServletRequest request, HttpServletResponse response) throws IOException {
    String redirectUrl = request.getParameter("redirectUrl");
    try {
      //解析token，并且单点登录
      String loginName = URLDecoder.decode(token, "UTF-8");
        UsernamePasswordToken upToken = new UsernamePasswordToken(loginName, "SSL");
      Subject subject = SecurityUtils.getSubject();
      subject.login(upToken);
    } catch (Exception e) {
      e.printStackTrace();
    }
    if (SecurityUtils.getSubject().getPrincipal() != null) {

      response.sendRedirect(redirectUrl);
    }
  }

  /**
   * 获取Ukey信息
   *
   * @param identityticket
   * @param challenge
   * @return
   */
  @ResponseBody
  @RequestMapping("/loginCA/getUserInfo")
  public Map<String, String> getUserInfo(String identityticket, String challenge, String verifyUrl) {
    Map<String, String> map = new HashMap<>();
    String username = "";
    String checkrest = "";//验证返回
    String url = verifyUrl;
    String resStr = "";
    try {
      //封装参数
      String soap = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
        + "<verifyidentityticketreq version=\"1\">"
        + "<challenge>" + challenge + "</challenge>"
        + "<identityticket>" + identityticket + "</identityticket>"
        + "<appserverid>337c7f2b-7a34-4f50-9141-bab9e6478cc8</appserverid>"
        + "</verifyidentityticketreq>";
      resStr = HttpUtil.post(url, soap);
      String getret = resStr.substring(resStr.indexOf("<result>") + 8, resStr.indexOf("</result>"));
      /*BASE64Decoder decoder = new BASE64Decoder();
      checkrest = new String(decoder.decodeBuffer(getret), "UTF-8");*/
      checkrest = Base64.decodeStr(getret, "UTF-8");
    } catch (Exception e) {
      e.printStackTrace();
    }
    map.put("xml", checkrest);
    logger.info("UKey信息：" + checkrest);
    String result = checkrest.substring(checkrest.indexOf("<result>") + 8, checkrest.indexOf("</result>"));
    if ("0".equals(result)) {
      username = checkrest.substring(checkrest.indexOf("<name>") + 6, checkrest.indexOf("</name>"));
      if ("宋清国".equals(username)) { //UKey内信息与数据库信息不匹配
        username = "黄若楠";
      }
      List<User> userList = authService.queryUserByName(username);
      if (CollUtil.isNotEmpty(userList)) {
        User u = userList.get(0);
        map.put("userId", u.getId());
        map.put("loginName", u.getLoginName());
      }
    }
    map.put("userName", username);
    return map;
  }

  /**
   * 临时存放密码
   *
   * @param password
   * @param session
   */
  @RequestMapping("/login/temp-pwd")
  @ResponseBody
  public void tempPwd(String password, HttpSession session) {
    session.setAttribute("TEMP_PASSWORD", password);
  }

  /**
   * ajax登录
   *
   * @param request
   * @param loginName
   * @param password
   * @return
   * @author: shen.shaohua
   * @since: 2019/10/25 17:17
   */
  @RequestMapping("/loginAjax")
  @ResponseBody
  public Map<String, String> loginAjax(HttpServletRequest request, String loginName, String password) {
    Map<String, String> mm = new HashMap<>();
    String status = "fail";
    String error = null;
    try {
      if (SecurityUtils.getSubject().getPrincipal() != null) {
        SecurityUtils.getSubject().logout();
      }
      String ssoLogin = request.getParameter("ssoLogin");
      if ("true".equals(ssoLogin)) { //使用ukey登录
        password = "useCaUkey";
      }
      UsernamePasswordToken upToken = new UsernamePasswordToken(loginName, password);
      Subject subject = SecurityUtils.getSubject();
      subject.login(upToken);
    } /*catch (AuthenticationException e) {
      e.printStackTrace();
      mm.put("errMsg", e.toString());
      mm.put("status", status);
      return mm;
    }*/
    catch (UnknownAccountException e) {
      error = "用户名或密码错误！";
      mm.put("errMsg",error);
      mm.put("status", status);
      return mm;
    } catch (IncorrectCredentialsException e) {
      error = "用户名或密码错误！";
      mm.put("errMsg",error);
      mm.put("status", status);
      return mm;
    } catch (ExcessiveAttemptsException e) {
      // TODO: handle exception
      error = "登录失败多次，账户锁定30分钟!";
      mm.put("errMsg",error);
      mm.put("status", status);
      return mm;
    } catch (AuthenticationException e) {
      // 其他错误，比如锁定，如果想单独处理请单独catch处理
      //error = "其他错误：" + e.getMessage();
      error = "用户名或密码错误！";
      mm.put("errMsg",error);
      mm.put("status", status);
      return mm;
    }

    IUser currentUser = (IUser) SecurityUtils.getSubject().getPrincipal();
    if (currentUser != null) {
      status = "success";
    }
    mm.put("status", status);
    return mm;
  }
}
