package com.cesgroup.bdc.logs.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cesgroup.bdc.logs.entity.LoginLog;
import com.cesgroup.bdc.logs.mapper.LoginLogMapper;
import com.cesgroup.bdc.logs.service.ILoginLogService;
import com.cesgroup.bdc.util.PrimaryKey;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.model.IUser;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * <p>
 * 登录日志 服务实现类
 * </p>
 *
 * @author shen.shaohua
 * @since 2019-07-24
 */
@Service
public class LoginLogServiceImpl extends ServiceImpl<LoginLogMapper, LoginLog> implements ILoginLogService {
  /**
   * 添加登录失败日志
   */
  @Override
  public void addFail(String loginName, String logDetail, String ip) {
    LoginLog loginLog = new LoginLog();
    loginLog.setId(PrimaryKey.generateId());
    loginLog.setIp(ip);
    loginLog.setLoginTime(new Date());
    loginLog.setLoginStatus("0");
    loginLog.setLogDetail(logDetail);

    loginLog.setUserName(loginName);
    this.save(loginLog);
  }

  /**
   * 添加登录失败日志
   */
  @Override
  public void addFail(IUser user, String logDetail, String ip) {
    LoginLog loginLog = new LoginLog();
    loginLog.setId(PrimaryKey.generateId());
    loginLog.setIp(ip);
    loginLog.setLoginTime(new Date());
    loginLog.setUserId(user.getId());
    loginLog.setUserName(user.getUserName());
    loginLog.setLoginStatus("0");
    loginLog.setLogDetail(logDetail);
    this.save(loginLog);
  }

  /**
   * 添加登录成功日志
   *
   * @param user
   * @param org
   * @param logDetail
   * @param ip
   */
  @Override
  public void addSuccess(IUser user, IOrganization org, String logDetail, String ip) {
    LoginLog loginLog = new LoginLog();
    loginLog.setId(PrimaryKey.generateId());
    loginLog.setIp(ip);
    loginLog.setLoginTime(new Date());
    loginLog.setUserId(user.getId());
    loginLog.setUserName(user.getUserName());
    loginLog.setLoginStatus("1");
    loginLog.setLogDetail(logDetail);

    loginLog.setDeptId(org.getId());
    loginLog.setDeptName(org.getName());

    this.save(loginLog);
  }
}
