package com.cesgroup.bdc.logs.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cesgroup.bdc.auth.service.SystemLogService;
import com.cesgroup.bdc.logs.entity.LogType;
import com.cesgroup.bdc.logs.entity.Logs;
import com.cesgroup.bdc.logs.mapper.LogsMapper;
import com.cesgroup.bdc.logs.service.ILogsService;
import com.cesgroup.bdc.util.IpUtil;
import com.cesgroup.bdc.util.PrimaryKey;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.service.AuthsystemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author shen.shaohua
 * @since 2019-07-02
 */
@Service
public class LogsServiceImpl extends ServiceImpl<LogsMapper, Logs> implements ILogsService {

  @Autowired
  private AuthsystemService authsystemService;

  /**
   * 用户管理平台日志
   */
  @Autowired
  private SystemLogService systemLogService;

  /**
   * @param request
   * @param userId:用户ID
   * @param userName:用户名
   * @param logType:操作类型
   * @param status:操作状态
   * @param operateObjectType:操作对象
   * @param errorMsg
   * @param module:操作模块
   * @param operateObject:操作对象类型
   * @return void
   * @Author ChenKai
   * @Description 新增日志
   * @Date 2019-07-19 13:59
   */
  @Override
  public void operateLog(HttpServletRequest request, String userId, String userName, LogType logType,
                         String status, String operateObjectType, String errorMsg, String module, String operateObject) {
    Logs logs = new Logs();
    IOrganization org = authsystemService.getOrganizationByUserId(userId);
    String ip = IpUtil.getIpAddr(request);
    logs.setId(PrimaryKey.generateId());
    logs.setCreateTime(new Date());
    logs.setUserId(userId);
    logs.setUserName(userName);
    logs.setIp(ip);
    logs.setUrl(request.getRequestURI());
    logs.setLogType(logType.toString());
    logs.setOrgId(org.getId());
    logs.setOrgName(org.getName());
    logs.setStatus(status);
    logs.setModule(module);
    logs.setOperateObject(operateObject);
    logs.setOperateObjectType(operateObjectType);
    if (logType == LogType.ADD) {
      logs.setOperate("在" + module + "中新增" + operateObjectType + "," + operateObjectType + "标题=" + operateObject);
    } else if (logType == LogType.DELETE) {
      logs.setOperate("在" + module + "中删除" + operateObjectType + "," + operateObjectType + "标题=" + operateObject);
    } else if (logType == LogType.MODIFY) {
      logs.setOperate("在" + module + "中修改" + operateObjectType + "," + operateObjectType + "标题=" + operateObject);
    } else if (logType == LogType.QUERY) {
      logs.setOperate("在" + module + "中查看" + operateObjectType + "," + operateObjectType + "标题=" + operateObject);
    } else if (logType == LogType.PRINT) {
      logs.setOperate("在" + module + "中打印" + operateObjectType + "," + operateObjectType + "标题=" + operateObject);
    } else if (logType == LogType.DOWNLOAD) {
      logs.setOperate("在" + module + "中下载" + operateObjectType + "," + operateObjectType + "标题=" + operateObject);
    }
    if (StrUtil.isNotEmpty(errorMsg)) {
      errorMsg = StrUtil.subSufByLength(errorMsg, 1000);
      logs.setErrorMsg(errorMsg);
    }
    this.save(logs);

    String message = logs.getOperate();
    if (StrUtil.isNotEmpty(errorMsg)) {
      message += " ERROR: " + logs.getErrorMsg();
    }
    systemLogService.addOperateLog(userId, userName, logs.getOrgId(), logs.getOrgName(), logs.getModule(), message, logs.getIp());
  }
}
