package com.cesgroup.bdc.mail.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import cn.hutool.poi.excel.StyleSet;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.cesgroup.bdc.apply.entity.ExpireApply;
import com.cesgroup.bdc.apply.service.IExpireApplyService;
import com.cesgroup.bdc.attachment.entity.RePrintApply;
import com.cesgroup.bdc.attachment.service.IRePrintApplyService;
import com.cesgroup.bdc.logs.entity.LogType;
import com.cesgroup.bdc.logs.service.ILogsService;
import com.cesgroup.bdc.mail.entity.MailReceive;
import com.cesgroup.bdc.mail.entity.MailSend;
import com.cesgroup.bdc.mail.service.IMailReceiveService;
import com.cesgroup.bdc.mail.service.IMailSendService;
import com.cesgroup.bdc.message.entity.MsgType;
import com.cesgroup.bdc.message.service.INewMessagesService;
import com.cesgroup.bdc.stamp.service.IStampService;
import com.cesgroup.bdc.util.Constant;
import com.cesgroup.bdc.util.CuiPage;
import com.cesgroup.bdc.util.DownloadUtil;
import com.cesgroup.bdc.util.PageVo;
import com.cesgroup.bdc.wdgl.vo.ConditionVo;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.model.IUser;
import com.cesgroup.kingkong.core.security.service.AuthsystemService;
import com.cesgroup.kingkong.web.BaseServiceController;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 收件信息 前端控制器
 * </p>
 *
 * @author shen.shaohua
 * @since 2019-07-08
 */
@Controller
@RequestMapping("/mail-receive")
public class MailReceiveController extends BaseServiceController<MailReceive, IMailReceiveService> {
  @Autowired
  private AuthsystemService authsystemService;

  @Autowired
  private IRePrintApplyService rePrintApplyService;

  @Autowired
  private ILogsService logsService;

  @Autowired
  private IMailSendService mailSendService;

  @Autowired
  private IExpireApplyService expireApplyService;

  @Autowired
  private INewMessagesService messagesService;

  @Autowired
  private IStampService iStampService;

  /**
   * 签收状态
   *
   * @param sendId 发件id
   * @author: shen.shaohua
   * @since: 2019/7/11 16:49
   */
  @RequestMapping("/viewList/{sendId}")
  public ModelAndView viewList(@PathVariable String sendId) {
    ModelAndView mv = new ModelAndView("mailReceive/viewList");
    mv.addObject("sendId", sendId);
    return mv;
  }

  /**
   * 签收状态分页查询
   *
   * @param sendId  发件idv
   * @param pager
   * @param request
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/11 17:47
   */
  @RequestMapping("/queryViewList/{sendId}")
  @ResponseBody
  public PageVo queryViewList(@PathVariable String sendId, CuiPage<MailReceive> pager, HttpServletRequest request) {
    QueryWrapper<MailReceive> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("mail_send_id", sendId);

    String deptName = request.getParameter("deptName");
    queryWrapper.like(StrUtil.isNotBlank(deptName), "dept_name", deptName);

    String receiveUserName = request.getParameter("receiveUserName");
    queryWrapper.like(StrUtil.isNotBlank(receiveUserName), "receive_user_name", receiveUserName);

    String status = request.getParameter("status");
    queryWrapper.eq(StrUtil.isNotBlank(status), "status", status);

    String dateBegin = request.getParameter("dateBegin");
    String dateEnd = request.getParameter("dateEnd");
    if (!StrUtil.isAllBlank(dateBegin, dateEnd)) {
      //queryWrapper.isNotNull("receive_date");
      if (StrUtil.isNotBlank(dateBegin)) {
        Date dateBeginD = DateUtil.parseDate(dateBegin);
        queryWrapper.ge("receive_date", dateBeginD);
      }
      if (StrUtil.isNotBlank(dateEnd)) {
        Date dateEndD = DateUtil.parseDate(dateEnd);
        queryWrapper.le("receive_date", dateEndD);
      }
    }
    queryWrapper.orderByAsc("to_number(SORT_NUMBER)");
    IPage<MailReceive> p = service.page(pager, queryWrapper);
    PageVo<MailReceive> pv = PageVo.<MailReceive>convertFromMybatisplusPage(p);
    return pv;
  }

  @RequestMapping("details/exp")
  public void detailsExp(String id, HttpServletRequest request, HttpServletResponse response) {
    ServletOutputStream sos = null;
    try {
      MailSend mailSend = mailSendService.getById(id);

      ExcelWriter writer = ExcelUtil.getWriter(true);
      StyleSet styleSet = writer.getStyleSet();
      styleSet.setBorder(BorderStyle.THIN, IndexedColors.BLUE);
      /*styleSet.setBorder(BorderStyle.MEDIUM, IndexedColors.BLACK);*/
      writer.setColumnWidth(-1, 30);
      writer.setColumnWidth(0, 30);
      writer.setColumnWidth(1, 60);
      writer.setColumnWidth(2, 30);
      writer.setColumnWidth(3, 50);
      //writer.setRowHeight(-1, 25);

      List<List<String>> rowList = new ArrayList<>();
      List<String> title = new ArrayList<>();
      title.add("文电信息");
      rowList.add(title);
      styleSet.setBorder(BorderStyle.THIN, IndexedColors.BLACK);
      /*styleSet.setBorder(BorderStyle.MEDIUM, IndexedColors.BLACK);*/
      writer.setColumnWidth(-1, 20);
      writer.setColumnWidth(0, 30);
      writer.setColumnWidth(1, 55);
      writer.setColumnWidth(2, 20);
      writer.setColumnWidth(3, 45);
      writer.setColumnWidth(4, 30);
      List<String> head = new ArrayList<>();
      ///head.add("序号");
      head.add("发文号");
      head.add("标题");
      head.add("类型");
      head.add("发送单位");
      head.add("发送时间");
      head.add("紧急度");
      rowList.add(head);
      List<String> row = new ArrayList<>();
      row.add(mailSend.getMailNo());//发文号
      row.add(mailSend.getMailTitle());//文电标题
      row.add(mailSend.getMailType());//类型
      row.add(mailSend.getDeptName());//发送单位
      row.add(DateUtil.formatDateTime(mailSend.getSendTime()));//发送时间
      row.add(mailSend.getEmergency());//紧急度
      rowList.add(row);
      List<String> head3 = new ArrayList<>();
      head3.add("有效时间");
      head3.add("页数");
      head3.add("发送人");
      head3.add("短信通知");
      head3.add("密级");
      head3.add("备注");
      rowList.add(head3);
      List<String> row3 = new ArrayList<>();
      row3.add(DateUtil.formatDateTime(mailSend.getEffDate()));//有效时间
      row3.add(NumberUtil.toStr(mailSend.getMailPage(), ""));//页数
      row3.add(mailSend.getUserName());//发送人
      row3.add("0".equals(mailSend.getNeedMsg()) ? "无" : "有");//短信通知
      row3.add(mailSend.getSafeLevel());//密级
      row3.add(mailSend.getRemark());//备注
      rowList.add(row3);
      rowList.add(new ArrayList<>());
      List<String> title2 = new ArrayList<>();
      title2.add("签收状态表");
      rowList.add(title2);
      QueryWrapper<MailReceive> queryWrapper = new QueryWrapper<>();
      queryWrapper.eq("mail_send_id", id);
      queryWrapper.orderByAsc("to_number(SORT_NUMBER)");
      List<MailReceive> mailReceiveList = this.getBaseService().list(queryWrapper);
      List<String> head2 = new ArrayList<>();
      ///head.add("序号");
      head2.add("接收单位");
      head2.add("签收人");
      head2.add("签收时间");
      head2.add("状态");
      head2.add("回执");
      rowList.add(head2);
      int i = 0;
      for (MailReceive m : mailReceiveList) {
        //i++;
        List<String> row2 = new ArrayList<>();
        ///row.add(i + "");//序号
        row2.add(m.getDeptName());//接受单位
        row2.add(m.getReceiveUserName());//签收人
        row2.add(DateUtil.formatDateTime(m.getReceiveDate()));//签收时间
        row2.add("1".equals(m.getStatus()) ? "未签收" : "已签收");//状态
        row2.add(m.getReplyText());//回执
        rowList.add(row2);
      }
      String fileName = "文电管理—签收状况" + DateUtil.today() + ".xlsx";
      //fileName = new String(fileName.getBytes("GBK"), "ISO-8859-1");
      sos = response.getOutputStream();
      response.setStatus(200);
      //response.setContentType("application/x-msdownload");
      //response.setContentType("application/vnd.ms-excel;charset=utf-8");
      response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
      response.setHeader("Content-Disposition", DownloadUtil.getContentDisposition(request, fileName));
      writer.write(rowList);
      writer.flush(sos);
      writer.close();
      //sos.flush();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * @return org.springframework.web.servlet.ModelAndView
   * @Author ChenKai
   * @Description 收件夹页面
   * @Date 2019-07-09 11:00
   */
  @RequestMapping("/receiveList/{status}")
  public ModelAndView receiveList(@PathVariable String status) {
    ModelAndView mv = new ModelAndView("mailReceive/receiveList");
    mv.addObject("status", status);
    return mv;
  }

  @RequestMapping("/queryReceiveList/{status}")
  @ResponseBody
  public PageVo query(CuiPage<MailReceive> pager, MailReceive mailReceive, @PathVariable String status) {
    IUser uu = getUser();
    if (status.equals("1")) {
      mailReceive.setStatus("1");
    } else {
      mailReceive.setStatus("");
    }
    IOrganization org = authsystemService.getOrganizationByUserId(uu.getId());
    mailReceive.setDeptId(org.getId());
    IPage<MailReceive> page = getBaseService().queryReceiveListPage(pager, mailReceive, null, null, null);
    PageVo<MailReceive> pv = PageVo.<MailReceive>convertFromMybatisplusPage(page);
    return pv;
  }

  @RequestMapping("/queryReceiveList2")
  @ResponseBody
  public PageVo query2(CuiPage<MailReceive> pager, MailReceive mailReceive, String mailTitle, String sendTimeStart, String sendTimeEnd) {
    IUser uu = getUser();
    IOrganization org = authsystemService.getOrganizationByUserId(uu.getId());
    mailReceive.setDeptId(org.getId());
    if ("A".equals(mailReceive.getStatus())) {
      mailReceive.setStatus("");
    }
    IPage<MailReceive> page = getBaseService().queryReceiveListPage(pager, mailReceive, mailTitle, sendTimeStart, sendTimeEnd);
    PageVo<MailReceive> pv = PageVo.<MailReceive>convertFromMybatisplusPage(page);
    return pv;
  }

  /**
   * @param id:表单id
   * @return org.springframework.web.servlet.ModelAndView
   * @Author ChenKai
   * @Description 跳转收件表单
   * @Date 2019-07-15 12:11
   */
  @RequestMapping("/edit")
  public ModelAndView edit(String id, String mailSendId) {
    ModelAndView mv = new ModelAndView("mailReceive/edit");
    mv.addObject("id", id);
    mv.addObject("mailSendId", mailSendId);
    String canPrintNot = mailSendService.getById(mailSendId).getCanDownload();
    mv.addObject("canPrintNot", canPrintNot);
    mv.addObject("currentOrgId", authsystemService.getOrganizationByUserId(getUser().getId()).getId());
    return mv;
  }

  /**
   * 查看页面
   *
   * @param id 收文id
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/16 10:25
   */
  @RequestMapping("/view/{id}")
  public ModelAndView edit(@PathVariable String id) {
    ModelAndView mv = new ModelAndView("mailReceive/view");
    mv.addObject("id", id);
    return mv;
  }

  /**
   * 查看页面
   *
   * @param id 收文id
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/16 10:25
   */
  @RequestMapping("/view/{mailReceiveId}/{id}")
  public ModelAndView edit2(@PathVariable String mailReceiveId, @PathVariable String id) {
    ModelAndView mv = new ModelAndView("mailReceive/view");
    mv.addObject("id", mailReceiveId);
    mv.addObject("applyId", id);
    return mv;
  }

  /**
   * 查看页面
   *
   * @param id 收文id
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/16 10:25
   */
  @RequestMapping("/load/{id}")
  @ResponseBody
  public MailReceive load(@PathVariable String id) {
    MailReceive mailReceive = getBaseService().selectById(id);

    return mailReceive;
  }

  @RequestMapping("/receiveOne/exp")
  public void receiveOneExp(String id, HttpServletResponse response, HttpServletRequest request) {
    ServletOutputStream sos = null;

    try {
      ExcelWriter writer = ExcelUtil.getWriter(true);
      StyleSet styleSet = writer.getStyleSet();
      styleSet.setBorder(BorderStyle.THIN, IndexedColors.BLACK);
      writer.setColumnWidth(-1, 20);
      writer.setColumnWidth(0, 20);
      writer.setColumnWidth(1, 25);
      writer.setColumnWidth(2, 55);
      writer.setColumnWidth(3, 25);
      writer.setColumnWidth(4, 20);
      writer.setColumnWidth(5, 45);
      //writer.setRowHeight(-1, 20);
      List<List<String>> rowList = new ArrayList<>();
      List<String> title = new ArrayList<>();
      title.add("文电信息");
      rowList.add(title);
      List<String> head = new ArrayList<>();
      head.add("发文号");
      head.add("有效时间");
      head.add("标题");
      head.add("类型");
      head.add("页数");
      head.add("发送单位");
      rowList.add(head);
      MailReceive mailReceive = getBaseService().selectById(id);
      MailSend s = mailSendService.getById(mailReceive.getMailSendId());
      List<String> row = new ArrayList<>();
      row.add(s.getMailNo());//发文号
      row.add(DateUtil.formatDateTime(s.getEffDate()));//有效时间
      row.add(s.getMailTitle());//标题
      row.add(s.getMailType());//类型
      row.add(NumberUtil.toStr(s.getMailPage(), ""));//页数
      row.add(s.getDeptName());//发文单位
      rowList.add(row);
      List<String> head2 = new ArrayList<>();
      head2.add("签收状况");
      head2.add("发送人");
      head2.add("签收人");
      head2.add("发送时间");
      head2.add("签收时间");
      head2.add("紧急度");
      rowList.add(head2);
      List<String> row2 = new ArrayList<>();
      row2.add("0".equals(s.getStatus()) ? "未完成" : "已完成");//签收状况
      row2.add(s.getUserName());//发送人
      row2.add(mailReceive.getReceiveUserName());//签收人
      row2.add(DateUtil.formatDateTime(s.getSendTime()));//发送时间
      row2.add(DateUtil.formatDateTime(mailReceive.getReceiveDate()));//签收时间
      row2.add(s.getEmergency());//紧急度
      rowList.add(row2);
      List<String> head3 = new ArrayList<>();
      head3.add("密级");
      head3.add("短信通知");
      head3.add("会议时间");
      head3.add("短信内容");
      head3.add("办理情况");
      head3.add("备注");
      rowList.add(head3);
      List<String> row3 = new ArrayList<>();
      row3.add(s.getSafeLevel());//密级
      row3.add("0".equals(s.getNeedMsg()) ? "无" : "有");//短信通知
      row3.add(DateUtil.formatDateTime(s.getPartyTime()));//会议时间
      row3.add(s.getMsgContent());//短信内容
      row3.add(null);//办理情况
      row3.add(s.getRemark());//备注
      rowList.add(row3);
      String fileName = "文电信息" + DateUtil.today() + ".xlsx";
      //fileName = new String(fileName.getBytes("GBK"), "ISO-8859-1");
      sos = response.getOutputStream();
      response.setStatus(200);
      //response.setContentType("application/x-msdownload");
      //response.setContentType("application/vnd.ms-excel;charset=utf-8");
      response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
      response.setHeader("Content-Disposition", DownloadUtil.getContentDisposition(request, fileName));
      writer.write(rowList);
      writer.flush(sos);
      writer.close();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * @param bussinessId:表单id
   * @return java.util.Map<java.lang.String, java.lang.String>
   * @Author ChenKai
   * @Description 查看收件表单
   * @Date 2019-07-15 12:55
   */
  @RequestMapping("/loadForm")
  @ResponseBody
  public MailReceive loadForm(String bussinessId, HttpServletRequest request) {
    IUser uu = this.getUser();
    MailReceive mailReceive = getBaseService().queryReceiveFormById(bussinessId);
    if (StrUtil.isBlank(mailReceive.getReceiveUserName())) {//如果签收人为空,就说明没有签收,填入当前登录人姓名
      mailReceive.setReceiveUserName(uu.getUserName());
    }
    QueryWrapper<ExpireApply> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("MAIL_RECEIVE_ID", bussinessId);
    queryWrapper.orderByDesc("APPLY_DATE");
    List<ExpireApply> list = expireApplyService.list(queryWrapper);
    if (list.size() > 0) {
      Date newDate = list.get(0).getExpireDate();
      mailReceive.getMailSend().setEffDate(newDate);
    }
    logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.QUERY, "成功", "文电", "", "收件箱", mailReceive.getMailSend().getMailTitle());
    return mailReceive;
  }

  /**
   * 归类统计
   *
   * @param conditionVo
   * @param session
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/23 17:14
   */
  @RequestMapping("/statistics")
  @ResponseBody
  public List<Map<String, Object>> statistics(ConditionVo conditionVo, HttpSession session) {
    IUser uu = getUser();
    conditionVo.setSendUserId(uu.getId());
    IOrganization org = getOrg(session);
    conditionVo.setReceiveDeptId(org.getId());
    List<Map<String, Object>> results = new ArrayList<>();
    String statistics = conditionVo.getStatistics();
    if (statistics.contains("0")) { //类型
      List<Map<String, Object>> mapList = getBaseService().statistics("MAIL_TYPE", conditionVo);
      results.addAll(mapList);
    }
    if (statistics.contains("1")) { //密级
      List<Map<String, Object>> mapList = getBaseService().statistics("SAFE_LEVEL", conditionVo);
      results.addAll(mapList);
    }
    if (statistics.contains("2")) { //紧急度
      List<Map<String, Object>> mapList = getBaseService().statistics("EMERGENCY", conditionVo);
      results.addAll(mapList);
    }
    return results;
  }

  /**
   * 文电管理分页查询
   *
   * @param pager
   * @param conditionVo
   * @param session
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/15 16:19
   */
  @RequestMapping("/toQuery")
  @ResponseBody
  public PageVo toQuery(CuiPage<MailReceive> pager, ConditionVo conditionVo, HttpSession session, HttpServletRequest request) {
    IUser uu = getUser();
    String status = request.getParameter("status");
    if (StrUtil.isNotBlank(status)) {
      conditionVo.setMailStatus(status);
    }
    conditionVo.setSendUserId(uu.getId());
    IOrganization org = getOrg(session);
    conditionVo.setReceiveDeptId(org.getId());
    IPage<MailReceive> p = getBaseService().queryPage(pager, conditionVo);
    PageVo<MailReceive> pv = PageVo.<MailReceive>convertFromMybatisplusPage(p);
    return pv;
  }

  /**
   * 导出文电管理收文excel
   *
   * @author: shen.shaohua
   * @since: 2019/7/16 15:44
   */
  @RequestMapping("/toQuery/exp")
  public void toQueryExp(ConditionVo conditionVo, HttpSession session, HttpServletResponse response, HttpServletRequest request) {
    ServletOutputStream sos = null;
    try {
      IUser uu = getUser();
      conditionVo.setSendUserId(uu.getId());
      IOrganization org = getOrg(session);
      conditionVo.setReceiveDeptId(org.getId());
      StringBuilder order = new StringBuilder();
      if (StrUtil.isNotEmpty(conditionVo.getDescs())) {
        String[] descs = conditionVo.getDescs().split(",");
        for (String desc : descs) {
          order.append("," + desc + " desc");
        }
      }
      if (StrUtil.isNotEmpty(conditionVo.getAscs())) {
        String[] ascs = conditionVo.getAscs().split(",");
        for (String asc : ascs) {
          order.append("," + asc + " asc");
        }
      }
      if (order.length() > 0) {
        conditionVo.setOrder(order.substring(1));
      }
      List<MailReceive> mailReceiveList = getBaseService().queryList(conditionVo);

      ExcelWriter writer = ExcelUtil.getWriter(true);
      StyleSet styleSet = writer.getStyleSet();
      styleSet.setBorder(BorderStyle.THIN, IndexedColors.BLACK);
      writer.setColumnWidth(-1, 30);
      writer.setColumnWidth(0, 6);
      writer.setColumnWidth(1, 30);
      writer.setColumnWidth(2, 50);
      writer.setColumnWidth(3, 60);
      //writer.setRowHeight(-1, 20);

      List<List<String>> rowList = new ArrayList<>();

      List<String> head = new ArrayList<>();
      head.add("序号");
      head.add("发文号");
      head.add("发文单位");
      head.add("标题");
      head.add("类型");
      head.add("紧急度");
      head.add("密级");
      head.add("页数");
      head.add("签收状态");
      head.add("发送时间");
      head.add("是否过期");
      head.add("收件单位");
      rowList.add(head);

      int i = 0;
      for (MailReceive r : mailReceiveList) {
        i++;
        MailSend s = r.getMailSend();
        List<String> row = new ArrayList<>();
        row.add(i + "");//序号
        row.add(s.getMailNo());//发文号
        row.add(s.getDeptName());//发文单位
        row.add(s.getMailTitle());//标题
        row.add(s.getMailType());//类型
        row.add(s.getEmergency());//紧急度
        row.add(s.getSafeLevel());//密级
        row.add(NumberUtil.toStr(s.getMailPage(), ""));//页数
        String status = r.getStatus();
        if (status.equals("1")) {
          status = "未签收";
        } else if (status.equals("2") || status.equals("3")) {
          status = "已签收";
        }
        ;
        row.add(status);//签收状态
        row.add(DateUtil.formatDateTime(s.getSendTime()));//发送时间
        Date today = DateUtil.parseDate(DateUtil.today());
        if (today.compareTo(s.getEffDate()) <= 0) {
          row.add("正常");//是否过期
        } else {
          row.add("过期");//是否过期
        }
        row.add(r.getDeptName());//收件单位

        rowList.add(row);
      }

      String fileName = "收文列表" + DateUtil.today() + ".xlsx";
      //fileName = new String(fileName.getBytes("GBK"), "ISO-8859-1");
      sos = response.getOutputStream();
      response.setStatus(200);
      //response.setContentType("application/x-msdownload");
      //response.setContentType("application/vnd.ms-excel;charset=utf-8");
      response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
      response.setHeader("Content-Disposition", DownloadUtil.getContentDisposition(request, fileName));
      writer.write(rowList);
      writer.flush(sos);
      writer.close();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * @param bussinessId:收件表单对应主键id
   * @param mailSendId:发件表单对应主键id
   * @param replyText:收件签收回执内容
   * @return boolean
   * @Author ChenKai
   * @Description 更新签收状态
   * @Date 2019-07-15 17:45
   */
  @RequestMapping("/updateReplyStatus")
  @ResponseBody
  public boolean updateReplyStatus(String bussinessId, String mailSendId, @RequestParam(value = "replyText", required = false) String replyText) {
    boolean flag = false;
    UpdateWrapper<MailReceive> updateReceiveWrapper = new UpdateWrapper<>();

    MailReceive mailReceive = new MailReceive();
    mailReceive.setStatus("2");
    if (StrUtil.isNotBlank(replyText)) {
      mailReceive.setReplyText(replyText);
    }
    IUser uu = getUser();
    mailReceive.setReceiveDate(new Date());
    mailReceive.setReceiveUserId(uu.getId());
    mailReceive.setReceiveUserName(uu.getUserName());
    updateReceiveWrapper.eq("ID", bussinessId);
    int updateResult = getBaseService().getBaseMapper().update(mailReceive, updateReceiveWrapper);
    /*
    QueryWrapper<MailReceive> queryReceiveWrapper = new QueryWrapper<>();
		queryReceiveWrapper.eq("MAIL_SEND_ID", mailSendId).eq("STATUS", "1");
		int count = getBaseService().getBaseMapper().selectCount(queryReceiveWrapper);
		if (count == 0) {
			MailSend mailSend = new MailSend();
			mailSend.setStatus("1");
			updateSendWrapper.eq("ID", mailSendId);
			mailSendService.getBaseMapper().update(mailSend, updateSendWrapper);
		}*/
    getBaseService().updateMailSendStatus(getBaseService().getById(bussinessId).getMailSendId());
    messagesService.updateMessageStatus(authsystemService.getOrganizationByUserId(this.getUser().getId()).getId(), bussinessId, MsgType.RECEIVE_MSG);

    if (updateResult > 0) {
      flag = true;
    }
    return flag;
  }

  /**
   * @return java.util.Map<java.lang.String, java.lang.String>
   * @Author ChenKai
   * @Description 首页待签收, 审批, 过期数据数量查询
   * @Date 2019-07-19 10:02
   */
  @RequestMapping("/getAllToBeDoneCounts")
  @ResponseBody
  public Map<String, String> getAllToBeDoneCounts(HttpSession session) {
    Map<String, String> retMap = new HashMap<>();
    IUser user = getUser();
    IOrganization org = authsystemService.getOrganizationByUserId(user.getId());
    //String s = session.getAttribute(Constant.CURRENT_ORG_ID).toString();
    QueryWrapper<MailReceive> querReceive = new QueryWrapper<>();
    querReceive.eq("DEPT_ID", org.getId()).eq("STATUS", "1");
    //int querReceiveCounts = getBaseService().count(querReceive);//待签数量
    int querReceiveCounts = getBaseService().querReceiveCounts(org.getId());//待签数量
    retMap.put("querReceiveCounts", "" + querReceiveCounts);
    QueryWrapper<RePrintApply> queryReprint = new QueryWrapper<>();
    queryReprint.eq("SEND_ATTACHMENT_DEPT_ID", org.getId()).eq("STATUS", "0");
    int queryReprintCounts = rePrintApplyService.count(queryReprint);//重打印审批
    retMap.put("queryReprintCounts", "" + queryReprintCounts);
    QueryWrapper<ExpireApply> queryExpireApply = new QueryWrapper<>();
    queryExpireApply.eq("APPROVE_DEPT_ID", org.getId()).eq("APPROVE_RESULT", "0");
    int queryExpireApplyCounts = expireApplyService.count(queryExpireApply);//过期数据审批
    retMap.put("queryExpireApplyCounts", "" + queryExpireApplyCounts);
    int wdCount=iStampService.getYzCount();
    retMap.put("queryStampApplyCounts", "" + wdCount);
    return retMap;
  }

  @RequestMapping("/getAllToBeDoneCounts4other")
  @ResponseBody
  public String getAllToBeDoneCounts4other(String deptId) {
    Map<String, String> retMap = new HashMap<>();
    QueryWrapper<MailReceive> querReceive = new QueryWrapper<>();
    querReceive.eq("DEPT_ID", deptId).eq("STATUS", "1");
    //int querReceiveCounts = getBaseService().count(querReceive);//待签数量
    int querReceiveCounts = getBaseService().querReceiveCounts(deptId);//待签数量
    retMap.put("querReceiveCounts", "" + querReceiveCounts);
    QueryWrapper<RePrintApply> queryReprint = new QueryWrapper<>();
    queryReprint.eq("SEND_ATTACHMENT_DEPT_ID", deptId).eq("STATUS", "0");
    int queryReprintCounts = rePrintApplyService.count(queryReprint);//重打印审批
    return querReceiveCounts + queryReprintCounts + "";
  }

  /**
   * 强制签收
   *
   * @param ids
   * @author: shen.shaohua
   * @since: 2019/7/17 15:04
   */
  @RequestMapping("/forceReceive")
  @ResponseBody
  public boolean forceReceive(@RequestParam("ids") String ids) {
    boolean flag = false;
    String[] idArr = ids.split(",");
    QueryWrapper<MailReceive> queryWrapper = new QueryWrapper<>();
    queryWrapper.in("ID", idArr).eq("STATUS", "1");
    List<MailReceive> list = getBaseService().list(queryWrapper);
    list.forEach(mailReceive -> {
      mailReceive.setStatus("3");
      mailReceive.setReceiveDate(new Date());
    });
    getBaseService().updateBatchById(list);
    list.forEach(mailReceive -> {
      getBaseService().updateMailSendStatus(mailReceive.getMailSendId());
    });
    flag = true;
    return flag;
  }

  /**
   * 获取各个签收状态的数量
   *
   * @param sendId
   * @author: shen.shaohua
   * @since: 2019/7/17 16:09
   */
  @RequestMapping("/countGroupByStatus/{sendId}")
  @ResponseBody
  public List<Map<String, Object>> countGroupByStatus(@PathVariable String sendId) {
    QueryWrapper<MailReceive> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("MAIL_SEND_ID", sendId).groupBy("STATUS").select("STATUS", "COUNT(*) C");
    List<Map<String, Object>> list = getBaseService().listMaps(queryWrapper);
    return list;
  }

  @RequestMapping("/batchSign")
  @ResponseBody
  public Map<String, String> batchSign(String[] relevanceIds) {
    Map<String, String> mm = new HashMap<>();
    QueryWrapper<MailReceive> queryWrapper = new QueryWrapper<>();
    queryWrapper.in("ID", relevanceIds);
    List<MailReceive> list = getBaseService().list(queryWrapper);
    list.forEach(mailReceive -> {
      mailReceive.setStatus("2");
    });
    boolean result = getBaseService().updateBatchById(list);
    if (result) {
      mm.put("code", "0");
    } else {
      mm.put("code", "-1");
    }
    return mm;
  }

  /**
   * 下载所有文件
   * @return
   */
  @RequestMapping("/downloadFiles")
  public void downloadFiles(HttpServletRequest request, HttpServletResponse response){
    getBaseService().download(request,response);
  }

}
