package com.cesgroup.bdc.mail.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import cn.hutool.poi.excel.StyleSet;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.cesgroup.bdc.attachment.entity.Attachment;
import com.cesgroup.bdc.attachment.entity.PrintNum;
import com.cesgroup.bdc.attachment.service.IAttachmentService;
import com.cesgroup.bdc.attachment.service.IPrintNumService;
import com.cesgroup.bdc.logs.entity.LogType;
import com.cesgroup.bdc.logs.service.ILogsService;
import com.cesgroup.bdc.mail.entity.MailReceive;
import com.cesgroup.bdc.mail.entity.MailSend;
import com.cesgroup.bdc.mail.service.IMailReceiveService;
import com.cesgroup.bdc.mail.service.IMailSendService;
import com.cesgroup.bdc.message.entity.MsgType;
import com.cesgroup.bdc.message.service.INewMessagesService;
import com.cesgroup.bdc.stamp.entity.Stamp;
import com.cesgroup.bdc.stamp.service.IStampService;
import com.cesgroup.bdc.util.CuiPage;
import com.cesgroup.bdc.util.DownloadUtil;
import com.cesgroup.bdc.util.PageVo;
import com.cesgroup.bdc.util.PrimaryKey;
import com.cesgroup.bdc.wdgl.vo.ConditionVo;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.model.IUser;
import com.cesgroup.kingkong.core.security.service.AuthsystemService;
import com.cesgroup.kingkong.web.BaseServiceController;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 发件信息 前端控制器
 * </p>
 *
 * @author shen.shaohua
 * @since 2019-07-08
 */
@Controller
@RequestMapping("/mail-send")
public class MailSendController extends BaseServiceController<MailSend, IMailSendService> {
  @Autowired
  private AuthsystemService authsystemService;

  @Autowired
  private IMailReceiveService mailReceiveService;

  @Autowired
  private IAttachmentService attachmentService;

  @Autowired
  private IPrintNumService printNumService;

  @Autowired
  private IStampService iStampService;

  private String attachModule = "mail";

  @Autowired
  private ILogsService logsService;

  @Autowired
  private INewMessagesService newMessagesService;

  /**
   * 发件夹列表页面
   *
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/8 17:09
   */
  @RequestMapping("/sendList")
  public ModelAndView sendList(HttpServletRequest request) {
    ModelAndView mv = new ModelAndView("mailSend/sendList");
    if (request.getParameter("title")!=null) {
      mv.addObject("title",request.getParameter("title"));
    }
    return mv;
  }

  /**
   * 发件夹列表分页查询
   *
   * @param pager
   * @param request
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/12 0:22
   */
  @RequestMapping("/queryList")
  @ResponseBody
  public PageVo queryList(CuiPage<MailSend> pager, MailSend mailSend,
                          HttpServletRequest request, HttpSession session) {
    QueryWrapper<MailSend> queryWrapper = new QueryWrapper<>();
    //queryWrapper.eq("USER_ID", getUser().getId());
    IOrganization org = getOrg(session);
    queryWrapper.eq("DEPT_ID", org.getId());
    String effective = request.getParameter("effective");//只查询未超期数据
    if (StrUtil.isBlank(effective)) {
      effective = "true";
    }
    if ("true".equals(effective)) {
      Date today = DateUtil.parseDate(DateUtil.today());
      queryWrapper.ge("EFF_DATE", today);
    }
    if (StringUtils.isNotBlank(mailSend.getStatus())) {
      queryWrapper.eq("STATUS", mailSend.getStatus());
    }
    if (StringUtils.isNotBlank(mailSend.getMailTitle())) {
      queryWrapper.like("MAIL_TITLE", mailSend.getMailTitle());
    }
    String sendTimeStart = request.getParameter("sendTimeStart");
    String sendTimeEnd = request.getParameter("sendTimeEnd");
    /*if (StringUtils.isNotBlank(sendTimeStart) && StringUtils.isNotBlank(sendTimeEnd)) {
      queryWrapper.between("SEND_TIME", sendTimeStart, sendTimeEnd);
    }*/
    if (StringUtils.isNotBlank(sendTimeStart)) {
      Date startDate = DateUtil.parseDate(sendTimeStart);
      queryWrapper.ge("SEND_TIME", startDate);
    }
    if (StringUtils.isNotBlank(sendTimeEnd)) {
      Date endDate = DateUtil.parseDate(sendTimeEnd);
      endDate = DateUtil.offsetDay(endDate, 1);
      queryWrapper.lt("SEND_TIME", endDate);
    }
    String orderBy = request.getParameter("orderBy");
    this.queryWrapperOrderBy(queryWrapper, orderBy);
    IPage<MailSend> p = getBaseService().page(pager, queryWrapper);
    PageVo<MailSend> pv = PageVo.<MailSend>convertFromMybatisplusPage(p);
    return pv;
  }

  /**
   * 排序
   *
   * @param queryWrapper
   * @param orderBy
   * @author: shen.shaohua
   * @since: 2019/10/31 10:08
   */
  private void queryWrapperOrderBy(QueryWrapper queryWrapper, String orderBy) {
    if (StrUtil.isBlank(orderBy)) {
      return;
    }
    String[] sorts = orderBy.split(",");
    for (String sort : sorts) {
      String[] px = sort.split("@");
      String zd = px[0];
      String sj = px[1];
      if ("desc".equalsIgnoreCase(sj)) {
        queryWrapper.orderBy(true, false, zd);
      } else {
        queryWrapper.orderBy(true, true, zd);
      }
    }
  }

  /**
   * 添加页面
   *
   * @author: shen.shaohua
   * @since: 2019/7/8 17:58
   */
  @RequestMapping("/add")
  public ModelAndView add() {
    ModelAndView mv = new ModelAndView("mailSend/add");
    return mv;
  }

  /**
   * 查看页面
   *
   * @author: shen.shaohua
   * @since: 2019/7/9 17:38
   */
  @RequestMapping("/edit/{id}")
  public ModelAndView edit(@PathVariable String id) {
    ModelAndView mv = new ModelAndView("mailSend/edit");
    mv.addObject("id", id);
    return mv;
  }

  /**
   * 查看页面
   *
   * @author: shen.shaohua
   * @since: 2019/7/17 9:34
   */
  @RequestMapping("/view/{id}")
  public ModelAndView view(@PathVariable String id) {
    ModelAndView mv = new ModelAndView("mailSend/view");
    mv.addObject("id", id);
    return mv;
  }

  //根据id查询收文的单位有哪几个
  @ResponseBody
  @RequestMapping("/getDeptNames/{id}")
  public List<MailReceive> getDeptNames(@PathVariable String id) {
    QueryWrapper<MailReceive> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("MAIL_SEND_ID", id);
    List<MailReceive> list = mailReceiveService.getBaseMapper().selectList(queryWrapper);
    return list;
  }

  /**
   * 发送
   *
   * @param entity
   * @param receiveDept     接收单位json数组字符串
   * @param tempRelevanceId 附件临时的关联id
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/8 21:56
   */
  @RequestMapping("/send")
  @ResponseBody
  public Map<String, String> send(MailSend entity, String receiveDept, String tempRelevanceId,
                                  HttpServletRequest request, HttpSession session) {
    Map<String, String> mm = new HashMap<>();
    IUser uu = this.getUser();
    IOrganization org = getOrg(session);
    Date now = new Date();
    entity.setId(PrimaryKey.generateId());
    entity.setUserId(uu.getId());
    entity.setUserName(uu.getUserName());
    entity.setDeptId(org.getId());
    entity.setDeptName(org.getName());
    entity.setSendTime(now);
    entity.setStatus("0");
    getBaseService().save(entity);

    attachmentService.updateTempRelevanceId(tempRelevanceId, entity.getId());

    QueryWrapper<Attachment> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("RELEVANCE_ID", entity.getId());
    List<Attachment> list = attachmentService.getBaseMapper().selectList(queryWrapper);
    JSONArray jsonArray = new JSONArray(receiveDept);
    List<MailReceive> mailReceiveList = new ArrayList<>();
    for (int i = 0; i < jsonArray.size(); i++) {
      JSONObject deptJson = (JSONObject) jsonArray.get(i);
      MailReceive mailReceive = new MailReceive();
      mailReceive.setId(PrimaryKey.generateId());
      mailReceive.setMailSendId(entity.getId());
      mailReceive.setDeptId(deptJson.getStr("deptId"));
      mailReceive.setDeptName(deptJson.getStr("deptName"));
      mailReceive.setSortNumber(deptJson.getStr("num"));
      mailReceive.setStatus("1");
      mailReceiveList.add(mailReceive);
      newMessagesService.addNewMessage(deptJson.getStr("deptId"), this.getUser().getId(), mailReceive.getId(), MsgType.RECEIVE_MSG);
      if (list.size() > 0) {
        for (Attachment att : list
          ) {
          PrintNum printNum = new PrintNum();
          printNum.setId(PrimaryKey.generateId());
          printNum.setMialReceiveId(mailReceive.getId());
          printNum.setAttachId(att.getId());
          printNum.setPrintNumb(entity.getPrintNum());
          printNumService.getBaseMapper().insert(printNum);
        }
      }
    }
    if (mailReceiveService.saveBatch(mailReceiveList)) {
      logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.ADD, "成功", "文电", "", "发件箱", entity.getMailTitle());
    } else {
      logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.ADD, "失败", "文电", "", "发件箱", entity.getMailTitle());

    }
    mm.put("status", "success");
    return mm;
  }

  /**
   * 转发
   *
   * @param receiveDept     接收单位json数组字符串
   * @return
   * @author: dxd
   * @since: 2023/6/12 10:35
   */
  @RequestMapping("/forwardSend")
  @ResponseBody
  public Map<String, String> forwardSend(String mailSendId, String receiveDept, HttpSession session) {
    MailSend entity = getBaseService().getById(mailSendId);
    Map<String, String> mm = new HashMap<>();
    IUser uu = this.getUser();
    IOrganization org = getOrg(session);
    Date now = new Date();
    if (entity.getForwordDept() == null || entity.getForwordDept().equals("")) {
      entity.setForwordDept(entity.getDeptName());
    }else {
      entity.setForwordDept(entity.getForwordDept());
    }
    entity.setId(PrimaryKey.generateId());
    entity.setUserId(uu.getId());
    entity.setUserName(uu.getUserName());
    entity.setDeptId(org.getId());
    entity.setDeptName(org.getName());
    entity.setSendTime(now);
    entity.setStatus("0");
    getBaseService().save(entity);

    attachmentService.addFromOld(attachModule, mailSendId, attachModule, entity.getId(), uu, org);

    QueryWrapper<Attachment> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("RELEVANCE_ID", entity.getId());
    List<Attachment> attachmentList = attachmentService.list(queryWrapper);

    List<MailReceive> mailReceiveList = null;
    //收件
    if (StrUtil.isNotEmpty(receiveDept)) {
      JSONArray jsonArray = new JSONArray(receiveDept);
      mailReceiveList = new ArrayList<>();
      for (int i = 0; i < jsonArray.size(); i++) {
        JSONObject deptJson = (JSONObject) jsonArray.get(i);
        MailReceive mailReceive = new MailReceive();
        mailReceive.setId(PrimaryKey.generateId());
        mailReceive.setMailSendId(entity.getId());
        mailReceive.setDeptId(deptJson.getStr("deptId"));
        mailReceive.setDeptName(deptJson.getStr("deptName"));
        mailReceive.setStatus("1");
        mailReceiveList.add(mailReceive);
        newMessagesService.addNewMessage(deptJson.getStr("deptId"), this.getUser().getId(), mailReceive.getId(), MsgType.RECEIVE_MSG);
      }
    }
    mailReceiveService.saveBatch(mailReceiveList);

    if (attachmentList != null && attachmentList.size() > 0) {
      for (MailReceive mailReceive : mailReceiveList) {
        for (Attachment att : attachmentList) {
          PrintNum printNum = new PrintNum();
          printNum.setId(PrimaryKey.generateId());
          printNum.setMialReceiveId(mailReceive.getId());
          printNum.setAttachId(att.getId());
          printNum.setPrintNumb(entity.getPrintNum());
          printNumService.getBaseMapper().insert(printNum);
        }
      }
    }
    mm.put("status", "success");
    return mm;
  }

  /**
   * 修改并发送
   *
   * @param entity
   * @param receiveDept     接收单位json数组字符串
   * @param tempRelevanceId 附件临时的关联id
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/8 21:56
   */
  @RequestMapping("/sends")
  @ResponseBody
  public Map<String, String> sends(MailSend entity, String receiveDept, String tempRelevanceId,
      HttpServletRequest request, HttpSession session) {
    Map<String, String> mm = new HashMap<>();
    IUser uu = this.getUser();
    IOrganization org = getOrg(session);
    Date now = new Date();
    if (entity.getId()==null||entity.getId().isEmpty()) {
      entity.setId(PrimaryKey.generateId());
    }else {
      if (StrUtil.isNotEmpty(receiveDept)) {
        mailReceiveService.delByMailSendId(entity.getId());
      }
    }
    entity.setUserId(uu.getId());
    entity.setUserName(uu.getUserName());
    entity.setDeptId(org.getId());
    entity.setDeptName(org.getName());
    entity.setSendTime(now);
    entity.setStatus("0");
    getBaseService().saveOrUpdate(entity);
    if (StrUtil.isNotEmpty(receiveDept)) {
      attachmentService.updateTempRelevanceId(tempRelevanceId, entity.getId());

      QueryWrapper<Attachment> queryWrapper = new QueryWrapper<>();
      queryWrapper.eq("RELEVANCE_ID", entity.getId());
      List<Attachment> list = attachmentService.getBaseMapper().selectList(queryWrapper);
      JSONArray jsonArray = new JSONArray(receiveDept);
      List<MailReceive> mailReceiveList = new ArrayList<>();
      for (int i = 0; i < jsonArray.size(); i++) {
        JSONObject deptJson = (JSONObject) jsonArray.get(i);
        MailReceive mailReceive = new MailReceive();
        mailReceive.setId(PrimaryKey.generateId());
        mailReceive.setMailSendId(entity.getId());
        mailReceive.setDeptId(deptJson.getStr("deptId"));
        mailReceive.setDeptName(deptJson.getStr("deptName"));
        mailReceive.setSortNumber(deptJson.getStr("num"));
        mailReceive.setStatus("1");
        mailReceiveList.add(mailReceive);
        newMessagesService.addNewMessage(deptJson.getStr("deptId"), this.getUser().getId(), mailReceive.getId(), MsgType.RECEIVE_MSG);
        if (list.size() > 0) {
          for (Attachment att : list
          ) {
            PrintNum printNum = new PrintNum();
            printNum.setId(PrimaryKey.generateId());
            printNum.setMialReceiveId(mailReceive.getId());
            printNum.setAttachId(att.getId());
            printNum.setPrintNumb(entity.getPrintNum());
            printNumService.getBaseMapper().insert(printNum);
          }
        }
      }
      if (mailReceiveService.saveBatch(mailReceiveList)) {
        logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.ADD, "成功", "文电", "", "发件箱", entity.getMailTitle());
      } else {
        logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.ADD, "失败", "文电", "", "发件箱", entity.getMailTitle());

      }
    }
    mm.put("status", "success");
    return mm;
  }

  /**
   * 列表中点击发送
   * @param entity
   * @param request
   * @param session
   * @return
   */
  @RequestMapping("/sends1")
  @ResponseBody
  public Map<String, String> sends1(MailSend entity, HttpServletRequest request, HttpSession session) {
    Map<String, String> mm = new HashMap<>();
    IUser uu = this.getUser();
    IOrganization org = getOrg(session);
    Date now = new Date();
    entity.setUserId(uu.getId());
    entity.setUserName(uu.getUserName());
    entity.setDeptId(org.getId());
    entity.setDeptName(org.getName());
    entity.setSendTime(now);
    entity.setStatus("0");
    boolean b = getBaseService().saveOrUpdate(entity);
    if (b) {
      logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.ADD, "成功", "文电", "", "发件箱", entity.getMailTitle());
    } else {
      logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.ADD, "失败", "文电", "", "发件箱", entity.getMailTitle());

    }
    mm.put("status", "success");
    return mm;
  }

  /**
   * 暂存
   *
   * @param entity
   * @param receiveDept     接收单位json数组字符串
   * @param tempRelevanceId 附件临时的关联id
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/8 21:56
   */
  @RequestMapping("/saves")
  @ResponseBody
  public Map<String, String> saves(MailSend entity, String receiveDept, String tempRelevanceId,
      HttpServletRequest request, HttpSession session) {
    Map<String, String> mm = new HashMap<>();
    IUser uu = this.getUser();
    IOrganization org = getOrg(session);
    Date now = new Date();
    if (entity.getId()==null||entity.getId().isEmpty()) {
      entity.setId(PrimaryKey.generateId());
    }else {
      if (StrUtil.isNotEmpty(receiveDept)) {
        mailReceiveService.delByMailSendId(entity.getId());
      }
    }
    entity.setUserId(uu.getId());
    entity.setUserName(uu.getUserName());
    entity.setDeptId(org.getId());
    entity.setDeptName(org.getName());
    entity.setSendTime(now);
    //-1 暂存，0未签收，1签收
    entity.setStatus("-1");
    getBaseService().saveOrUpdate(entity);
    if (StrUtil.isNotEmpty(receiveDept)) {
      attachmentService.updateTempRelevanceId(tempRelevanceId, entity.getId());

      QueryWrapper<Attachment> queryWrapper = new QueryWrapper<>();
      queryWrapper.eq("RELEVANCE_ID", entity.getId());
      List<Attachment> list = attachmentService.getBaseMapper().selectList(queryWrapper);
      JSONArray jsonArray = new JSONArray(receiveDept);
      List<MailReceive> mailReceiveList = new ArrayList<>();
      for (int i = 0; i < jsonArray.size(); i++) {
        JSONObject deptJson = (JSONObject) jsonArray.get(i);
        MailReceive mailReceive = new MailReceive();
        mailReceive.setId(PrimaryKey.generateId());
        mailReceive.setMailSendId(entity.getId());
        mailReceive.setDeptId(deptJson.getStr("deptId"));
        mailReceive.setDeptName(deptJson.getStr("deptName"));
        mailReceive.setSortNumber(deptJson.getStr("num"));
        mailReceive.setStatus("1");
        mailReceiveList.add(mailReceive);
        newMessagesService.addNewMessage(deptJson.getStr("deptId"), this.getUser().getId(), mailReceive.getId(), MsgType.RECEIVE_MSG);
        if (list.size() > 0) {
          for (Attachment att : list
          ) {
            PrintNum printNum = new PrintNum();
            printNum.setId(PrimaryKey.generateId());
            printNum.setMialReceiveId(mailReceive.getId());
            printNum.setAttachId(att.getId());
            printNum.setPrintNumb(entity.getPrintNum());
            printNumService.getBaseMapper().insert(printNum);
          }
        }
      }
      if (mailReceiveService.saveBatch(mailReceiveList)) {
        logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.ADD, "成功", "文电", "", "发件箱", entity.getMailTitle());
      } else {
        logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.ADD, "失败", "文电", "", "发件箱", entity.getMailTitle());

      }
    }
    String id = entity.getId();
    mm.put("status", "success");
    mm.put("id", id);
    return mm;
  }

  /**
   * 再次发送
   *
   * @param entity
   * @param receiveDept 接收单位json数组字符串
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/11 17:52
   */
  @RequestMapping("/sendAgain")
  @ResponseBody
  public Map<String, String> sendAgain(MailSend entity, String receiveDept, HttpSession session) {
    Map<String, String> mm = new HashMap<>();
    IUser uu = this.getUser();
    IOrganization org = getOrg(session);
    Date now = new Date();
    entity.setId(PrimaryKey.generateId());
    entity.setUserId(uu.getId());
    entity.setUserName(uu.getUserName());
    entity.setDeptId(org.getId());
    entity.setDeptName(org.getName());
    entity.setSendTime(now);
    entity.setStatus("0");
    getBaseService().save(entity);

    attachmentService.addFromOld(attachModule, entity.getPreviousId(), attachModule, entity.getId(), uu, org);

    QueryWrapper<Attachment> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("RELEVANCE_ID", entity.getId());
    List<Attachment> attachmentList = attachmentService.list(queryWrapper);

    List<MailReceive> mailReceiveList = null;
    //收件
    if (StrUtil.isNotEmpty(receiveDept)) {
      JSONArray jsonArray = new JSONArray(receiveDept);
      mailReceiveList = new ArrayList<>();
      for (int i = 0; i < jsonArray.size(); i++) {
        JSONObject deptJson = (JSONObject) jsonArray.get(i);
        MailReceive mailReceive = new MailReceive();
        mailReceive.setId(PrimaryKey.generateId());
        mailReceive.setMailSendId(entity.getId());
        mailReceive.setDeptId(deptJson.getStr("deptId"));
        mailReceive.setDeptName(deptJson.getStr("deptName"));
        mailReceive.setStatus("1");
        mailReceiveList.add(mailReceive);
        newMessagesService.addNewMessage(deptJson.getStr("deptId"), this.getUser().getId(), mailReceive.getId(), MsgType.RECEIVE_MSG);
      }
    } else { //默认未签收单位
      mailReceiveList = mailReceiveService.findBySendIdInStatus(entity.getPreviousId(), "1");
      mailReceiveList.forEach(mailReceive -> {
        mailReceive.setId(PrimaryKey.generateId());
        mailReceive.setMailSendId(entity.getId());
        newMessagesService.addNewMessage(mailReceive.getDeptId(), this.getUser().getId(), mailReceive.getId(), MsgType.RECEIVE_MSG);

      });
    }
    mailReceiveService.saveBatch(mailReceiveList);

    if (attachmentList != null && attachmentList.size() > 0) {
      for (MailReceive mailReceive : mailReceiveList) {
        for (Attachment att : attachmentList) {
          PrintNum printNum = new PrintNum();
          printNum.setId(PrimaryKey.generateId());
          printNum.setMialReceiveId(mailReceive.getId());
          printNum.setAttachId(att.getId());
          printNum.setPrintNumb(entity.getPrintNum());
          printNumService.getBaseMapper().insert(printNum);
        }
      }
    }
    mm.put("status", "success");
    return mm;
  }

  /**
   * 删除，连同收件及附件
   *
   * @param ids 以,分隔
   * @author: shen.shaohua
   * @since: 2019/7/12 0:05
   */
  @RequestMapping("/del")
  @ResponseBody
  public boolean del(@RequestParam(value = "ids") String ids, HttpServletRequest request) {
    IUser uu = this.getUser();
    boolean b = false;
    String[] idArr = StrUtil.split(ids, ",");
    for (String id : idArr) {
      mailReceiveService.delByMailSendId(id);
      attachmentService.deleteByModuleAndRelevanceId(attachModule, id);
      String title = getBaseService().getById(id).getMailTitle();
      //根据发文id查询用章id，删除用章信息
      QueryWrapper<Stamp> queryWrapper1 = new QueryWrapper<>();
      queryWrapper1.eq("MAIL_SEND_ID", id);
      List<Stamp> list = iStampService.getBaseMapper().selectList(queryWrapper1);
      for (int i = 0; i <list.size() ; i++) {
        iStampService.deleteStampAndPHis(list.get(i).getId());
      }
      if (getBaseService().removeById(id)) {
        logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.DELETE, "成功", "文电", "", "发件箱", title);
      } else {
        logsService.operateLog(request, uu.getId(), uu.getUserName(), LogType.DELETE, "失败", "文电", "", "发件箱", title);

      }
    }
    b = true;
    return b;
  }

  /**
   * 归类统计
   *
   * @param conditionVo
   * @param request
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/23 16:48
   */
  @RequestMapping("/statistics")
  @ResponseBody
  public List<Map<String, Object>> statistics(ConditionVo conditionVo, HttpServletRequest request,HttpSession session) {
    QueryWrapper<MailSend> queryWrapper = this.createQueryWrapper(conditionVo, request,session);
    String statistics = conditionVo.getStatistics();
    List<Map<String, Object>> results = new ArrayList<>();
    if (statistics.contains("0")) { //类型
      QueryWrapper<MailSend> qw = queryWrapper.clone();
      qw.groupBy("MAIL_TYPE");
      qw.select("MAIL_TYPE", "COUNT(*) CC");
      List<Map<String, Object>> mapList = getBaseService().listMaps(qw);
      results.addAll(mapList);
    }
    if (statistics.contains("1")) { //密级
      QueryWrapper<MailSend> qw = queryWrapper.clone();
      qw.groupBy("SAFE_LEVEL");
      qw.select("SAFE_LEVEL", "COUNT(*) CC");
      List<Map<String, Object>> mapList = getBaseService().listMaps(qw);
      results.addAll(mapList);
    }
    if (statistics.contains("2")) { //紧急度
      QueryWrapper<MailSend> qw = queryWrapper.clone();
      qw.groupBy("EMERGENCY");
      qw.select("EMERGENCY", "COUNT(*) CC");
      List<Map<String, Object>> mapList = getBaseService().listMaps(qw);
      results.addAll(mapList);
    }
    return results;
  }

  /**
   * 文电管理分页查询
   *
   * @param pager
   * @param request
   * @return
   * @author: shen.shaohua
   * @since: 2019/7/15 14:31
   */
  @RequestMapping("/toQuery")
  @ResponseBody
  public PageVo toQuery(CuiPage<MailSend> pager, ConditionVo conditionVo, HttpServletRequest request,HttpSession session) {
    QueryWrapper<MailSend> queryWrapper = this.createQueryWrapper(conditionVo, request,session);
    IPage<MailSend> p = getBaseService().page(pager, queryWrapper);
    PageVo<MailSend> pv = PageVo.<MailSend>convertFromMybatisplusPage(p);
    return pv;
  }

  /**
   * 导出文电管理发文excel
   *
   * @param conditionVo
   * @param request
   * @param response
   */
  @RequestMapping("/toQuery/exp")
  public void toQueryExp(ConditionVo conditionVo, HttpServletRequest request, HttpServletResponse response,HttpSession session) {
    ServletOutputStream sos = null;
    try {
      QueryWrapper<MailSend> queryWrapper = this.createQueryWrapper(conditionVo, request, session);
      if (StrUtil.isNotEmpty(conditionVo.getDescs())) queryWrapper.orderByDesc(conditionVo.getDescs().split(","));
      if (StrUtil.isNotEmpty(conditionVo.getAscs())) queryWrapper.orderByAsc(conditionVo.getAscs().split(","));

      List<MailSend> mailSendList = getBaseService().list(queryWrapper);

      ExcelWriter writer = ExcelUtil.getWriter(true);
      StyleSet styleSet = writer.getStyleSet();
      styleSet.setBorder(BorderStyle.THIN, IndexedColors.BLACK);
      writer.setColumnWidth(-1, 30);
      writer.setColumnWidth(0, 6);
      writer.setColumnWidth(1, 30);
      writer.setColumnWidth(1, 60);
      //writer.setRowHeight(-1, 25);

      List<List<String>> rowList = new ArrayList<>();

      List<String> head = new ArrayList<>();
      head.add("序号");
      head.add("发文号");
      head.add("文电标题");
      head.add("类型");
      head.add("紧急度");
      head.add("密级");
      head.add("页数");
      head.add("发送时间");
      head.add("状态");
      rowList.add(head);

      int i = 0;
      for (MailSend m : mailSendList) {
        i++;
        List<String> row = new ArrayList<>();
        row.add(i + "");//序号
        row.add(m.getMailNo());//发文号
        row.add(m.getMailTitle());//文电标题
        row.add(m.getMailType());//类型
        row.add(m.getEmergency());//紧急度
        row.add(m.getSafeLevel());//密级
        row.add(NumberUtil.toStr(m.getMailPage(), ""));//页数
        row.add(DateUtil.formatDateTime(m.getSendTime()));//发送时间
        if ("0".equals(m.getStatus())){
          row.add("未完成");//状态
        }else if ("1".equals(m.getStatus())){
          row.add("完成");//状态
        }else if ("-1".equals(m.getStatus())){
          row.add("未发送");//状态
        }

        rowList.add(row);
      }
      String fileName = "发文列表" + DateUtil.today() + ".xlsx";
      //fileName = new String(fileName.getBytes("GBK"), "ISO-8859-1");
      sos = response.getOutputStream();
      response.setStatus(200);
      //response.setContentType("application/x-msdownload");
      //response.setContentType("application/vnd.ms-excel;charset=utf-8");
      response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
      response.setHeader("Content-Disposition", DownloadUtil.getContentDisposition(request, fileName));
      writer.write(rowList);
      writer.flush(sos);
      writer.close();
      //sos.flush();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  private QueryWrapper<MailSend> createQueryWrapper(ConditionVo conditionVo, HttpServletRequest request,HttpSession session) {
    QueryWrapper<MailSend> queryWrapper = new QueryWrapper<>();
    //Date today = DateUtil.parseDate(DateUtil.formatDate(new Date()));
    if (!"all".equals(conditionVo.getFlag())) {
        queryWrapper.eq("DEPT_ID", getOrg(session).getId());
        //queryWrapper.eq("user_id", getUser().getId());

    }
    queryWrapper.eq(StrUtil.isNotEmpty(conditionVo.getMailType()), "MAIL_TYPE", conditionVo.getMailType())
      .eq(StrUtil.isNotEmpty(conditionVo.getSafeLevel()), "SAFE_LEVEL", conditionVo.getSafeLevel())
      .eq(StrUtil.isNotEmpty(conditionVo.getEmergency()), "EMERGENCY", conditionVo.getEmergency())
      .eq(StrUtil.isNotEmpty(conditionVo.getMailStatus()), "STATUS", conditionVo.getMailStatus());
    if (conditionVo.getEffDateBegin() != null) { //有效时间-开始
      queryWrapper.ge("EFF_DATE", conditionVo.getEffDateBegin());
    }
    if (conditionVo.getEffDateEnd() != null) { //有效时间-结束
      queryWrapper.le("EFF_DATE", conditionVo.getEffDateEnd());
    }
    if (conditionVo.getSendTimeBegin() != null) { //发送时间-开始
      queryWrapper.ge("SEND_TIME", conditionVo.getSendTimeBegin());
    }
    if (conditionVo.getSendTimeEnd() != null) { //发送时间-结束
      queryWrapper.le("SEND_TIME", conditionVo.getSendTimeEnd());
    }

    QueryWrapper<MailReceive> mrQuery = new QueryWrapper<>();
    if (conditionVo.getReceiveDateBegin() != null || conditionVo.getReceiveDateEnd() != null) { //签收时间
      if (conditionVo.getReceiveDateBegin() != null && conditionVo.getReceiveDateEnd() == null) {
        mrQuery.notIn("STATUS", "1").ge("RECEIVE_DATE", conditionVo.getReceiveDateBegin());
      } else if (conditionVo.getReceiveDateBegin() == null && conditionVo.getReceiveDateEnd() != null) {
        mrQuery.notIn("STATUS", "1").le("RECEIVE_DATE", conditionVo.getReceiveDateEnd());
      } else if (conditionVo.getReceiveDateBegin() != null && conditionVo.getReceiveDateEnd() != null) {
        mrQuery.notIn("STATUS", "1").ge("RECEIVE_DATE", conditionVo.getReceiveDateBegin()).le("RECEIVE_DATE", conditionVo.getReceiveDateEnd());
      }
      List<MailReceive> list = mailReceiveService.list(mrQuery);
      List<String> sendIds = new ArrayList<>();
      for (MailReceive mailrec : list) {
        sendIds.add(mailrec.getMailSendId());
      }
      if (list.size() > 0 && list != null) {
        queryWrapper.in("ID", sendIds);
      } else {
        queryWrapper.eq("ID", "木有哦");
      }
    }
    if (StrUtil.isNotBlank(conditionVo.getReceiveDept())) { //接收单位
      mrQuery.like("DEPT_NAME", conditionVo.getReceiveDept());
      List<MailReceive> list = mailReceiveService.list(mrQuery);
      List<String> sendIds = new ArrayList<>();
      for (MailReceive mailrec : list) {
        sendIds.add(mailrec.getMailSendId());
      }
      if (CollUtil.isNotEmpty(list)) {
        queryWrapper.in("ID", sendIds);
      } else {
        queryWrapper.eq("ID", "木有哦");
      }
    }
    if (StrUtil.isNotBlank(conditionVo.getFileKeyWord())) { //全文检索
      String keyword = conditionVo.getFileKeyWord().replace("'", "''");
      /*List<String> sendIds = attachmentService.getRelevanceIds(attachModule, keyword);
      if (sendIds != null && !sendIds.isEmpty()) {
				queryWrapper.in("ID", sendIds);
			}*/
      String originalName = keyword;
      originalName = originalName.replace("/", "//").replace("_", "/_").replace("%", "/%");
      originalName = "%" + originalName + "%";
      String existsSql = "select a.RELEVANCE_ID from ATTACHMENT a,DISK_FILE d where a.DISK_FILE_ID=d.ID" +
        " and a.MODULE='" + attachModule + "' and a.RELEVANCE_ID = MAIL_SEND.id" +
        " and (d.ORIGINAL_NAME like '" + originalName + "' escape '/' or CONTAINS(d.TEXT, '" + keyword + "'))";
      queryWrapper.exists(existsSql);
    }
    queryWrapper.like(StrUtil.isNotEmpty(conditionVo.getSendDept()), "DEPT_NAME", conditionVo.getSendDept()); //发送单位
    queryWrapper.like(StrUtil.isNotEmpty(conditionVo.getMailNo()), "MAIL_NO", conditionVo.getMailNo()); //发文号
    queryWrapper.like(StrUtil.isNotEmpty(conditionVo.getMailTitle()), "MAIL_TITLE", conditionVo.getMailTitle()); //标题
    queryWrapper.like(StrUtil.isNotEmpty(conditionVo.getAutoDownload()), "CAN_DOWNLOAD", conditionVo.getAutoDownload()); //自动下载
    return queryWrapper;
  }
}
