package com.cesgroup.bdc.print.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import cn.hutool.poi.excel.StyleSet;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.cesgroup.bdc.driverManage.entity.DriversManage;
import com.cesgroup.bdc.print.entity.PrintRecord;
import com.cesgroup.bdc.print.service.IPrintRecordService;
import com.cesgroup.bdc.print.vo.PrintShowVo;
import com.cesgroup.bdc.util.CuiPage;
import com.cesgroup.bdc.util.DownloadUtil;
import com.cesgroup.bdc.util.PageVo;
import com.cesgroup.kingkong.web.BaseServiceController;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 打印记录 前端控制器
 * </p>
 *
 * @author shen.shaohua
 * @since 2019-07-26
 */
@Controller
@RequestMapping("/print-record")
public class PrintRecordController extends BaseServiceController<PrintRecord, IPrintRecordService> {
    /**
     * 打印查询列表页面
     */
    @RequestMapping("/index")
    public ModelAndView index() {
        ModelAndView mv = new ModelAndView("printRecord/index");
        return mv;
    }

    @RequestMapping("/indexQuery")
    @ResponseBody
    public PageVo<PrintShowVo> indexQuery(CuiPage<PrintShowVo> page, HttpServletRequest request) {
        IPage<PrintShowVo> p = getBaseService().printStatistics(page);
        PageVo<PrintShowVo> pv = PageVo.convertFromMybatisplusPage(p);
        return pv;
    }

    /**
     * 明细列表
     *
     * @return
     */
    @RequestMapping("/detailList")
    public ModelAndView detailList(String deptId, String attachmentId) {
        ModelAndView mv = new ModelAndView("printRecord/detailList");
        mv.addObject("deptId", deptId);
        mv.addObject("attachmentId", attachmentId);
        return mv;
    }


    @RequestMapping("/queryPrintDetail")
    @ResponseBody
    public PageVo<PrintRecord> queryPrintDetail(CuiPage<PrintRecord> page, HttpServletRequest request) {
        String deptId = request.getParameter("deptId");
        String attachmentId = request.getParameter("attachmentId");
        QueryWrapper<PrintRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("PRINT_TIME");
        queryWrapper.eq("DEPT_ID", deptId).eq("ATTACHMENT_ID", attachmentId);
        IPage<PrintRecord> p = getBaseService().page(page, queryWrapper);
        PageVo<PrintRecord> pv = PageVo.convertFromMybatisplusPage(p);
        return pv;
    }

    @RequestMapping("/queryByKeyWords")
    @ResponseBody
    public PageVo<PrintShowVo> queryByKeyWords(CuiPage<PrintShowVo> page, PrintShowVo printShowVo) {
        IPage<PrintShowVo> p = getBaseService().queryByKeyWords(page,printShowVo);
        PageVo<PrintShowVo> pv = PageVo.convertFromMybatisplusPage(p);
        return pv;
    }



    @RequestMapping("/indexQuery/exp")
    @ResponseBody
    public void toQueryExp(HttpServletRequest request, HttpServletResponse response) {
        ServletOutputStream sos = null;
        try {

            List<PrintShowVo> driverList = getBaseService().getAll();

            ExcelWriter writer = ExcelUtil.getWriter(true);
            StyleSet styleSet = writer.getStyleSet();
            styleSet.setBorder(BorderStyle.THIN, IndexedColors.BLACK);
            writer.setColumnWidth(-1, 30);
            writer.setColumnWidth(0, 6);
            writer.setColumnWidth(1, 50);
            writer.setColumnWidth(2, 30);
            writer.setColumnWidth(3, 50);
            //writer.setRowHeight(-1, 25);

            List<List<String>> rowList = new ArrayList<>();

            List<String> head = new ArrayList<>();
            head.add("序号");
            head.add("文件名");
            head.add("文号");
            head.add("接收单位");
            head.add("分数");
            head.add("已打印份数");
            rowList.add(head);

            int i = 0;
            for (PrintShowVo m : driverList) {
                i++;
                List<String> row = new ArrayList<>();
                row.add(i + "");//序号
                row.add(m.getFileName());//文件名
                row.add(m.getMailNo());//文号
                row.add(m.getDeptName());//接收单位
                row.add(m.getPrintNum());//份数
                row.add(m.getAlreadyPrintNum());//已打印份数
                rowList.add(row);
            }
            String fileName = "打印记录" + DateUtil.today() + ".xlsx";
            //fileName = new String(fileName.getBytes("GBK"), "ISO-8859-1");
            sos = response.getOutputStream();
            response.setStatus(200);
            //response.setContentType("application/x-msdownload");
            //response.setContentType("application/vnd.ms-excel;charset=utf-8");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setHeader("Content-Disposition", DownloadUtil.getContentDisposition(request, fileName));
            writer.write(rowList);
            writer.flush(sos);
            writer.close();
            //sos.flush();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


}
