package com.cesgroup.bdc.wfself.engine.service;

import com.cesgroup.bdc.wfself.entity.ProcInstHisInfo;
import com.cesgroup.bdc.wfself.entity.ProcInstRunVar;
import com.cesgroup.bdc.wfself.entity.TaskRunInfo;
import com.cesgroup.bdc.wfself.entity.TaskRunPartic;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.model.IUser;

import java.util.List;

/**
 * 历史任务信息mapper
 * @author BeiLang
 */
public interface WorkflowDao {

    /**
     * 根据任务信息的执行id，获取对应的历史记录信息
     *
     * @param execId
     * @return
     */
    ProcInstHisInfo queryProcInstHisInfo(String execId);

    /**
     * 根据的执行id 查询历史流程实例数据
     *
     * @param execIdList
     *            执行id List
     * @return
     */
    List<ProcInstHisInfo> queryProcInstHisInfoList(List<String> execIdList);

    /**
     * 根据流程实例id、节点code查询，排除退回操作节点
     *
     * @param procInstId
     *            流程实例id
     * @param nodeCode
     *            节点code
     * @return
     */
    List<ProcInstHisInfo> queryProcInstHisInfoList(String procInstId, String nodeCode);

    /**
     * 根据id获取TaskRunInfo
     *
     * @param id
     *            主键id
     * @return
     */
    TaskRunInfo queryTaskRunInfo(String id);

    /**
     * 根据流程实例id、节点code查询
     *
     * @param procInstId
     *            流程实例id
     * @param taskCode
     *            节点code
     * @return
     */
    List<TaskRunInfo> queryTaskRunInfoList(String procInstId, String taskCode);

    /**
     * 根据流程实例id、节点code查询
     *
     * @param procInstId
     *            流程实例id
     * @return
     */
    List<TaskRunInfo> queryTaskRunInfoList(String procInstId);

    /**
     * 根据实例id和任务code，获取参与者正序的List
     *
     * @param procInstId
     *            流程实例id
     * @param taskCode
     *            任务code
     * @return
     */
    List<TaskRunPartic> queryTaskRunParticList(String procInstId, String taskCode);


    /**
     * 根据实例id，查询参与人
     *
     * @param procInstId
     *            流程实例id
     * @return
     */
    List<TaskRunPartic> queryTaskRunParticList(String procInstId);

    /**
     * 查询流程实例变量
     *
     * @param procInstId
     *            流程实例id
     * @param nodeCode
     *            流程节点
     * @param name
     *            流程参数名
     * @return
     */
     ProcInstRunVar queryProcInstRunVar(String procInstId, String nodeCode, String name);

    /**
     * 查询流程实例变量
     *
     * @param procInstId
     *            流程实例id
     * @param name
     *            流程参数名
     * @param data
     *            流程参数值
     * @return
     */
     List<ProcInstRunVar> queryProcInstRunVarList(String procInstId, String name, String data);

    /**
     * 查询流程实例变量
     *
     * @param procInstId
     *            流程实例id
     * @param nodeCode
     *            流程节点
     * @return
     */
    List<ProcInstRunVar> queryProcInstRunVarList(String procInstId, String nodeCode);

    /**
     * 查询流程实例变量
     *
     * @param procInstId
     *            流程实例id
     * @return
     */
    List<ProcInstRunVar> queryProcInstRunVarList(String procInstId);

    /**
     * 是否指定流程实例，节点任务已完成
     *
     * @param procInstId
     * @param nodeCode
     * @return
     */

    /**
     * 是否指定流程实例，未完成的只存在一个，并且执行id与之相同，或完成
     *
     * @param procInstId
     *            流程实例id
     * @param execId
     *            执行id
     * @param nodeCode
     *            流程节点
     * @return
     */
    public boolean isUniqueOrCompleteUserTask(String procInstId, String execId, String nodeCode);

    /**
     * 新增数据
     *
     * @param entity
     */
    public void save(Object entity);

    /**
     * 更新数据
     *
     * @param entity
     */
    public void update(Object entity);

    /**
     * 删除数据
     *
     */
    public void remove(Object entity);

    /**
     * 删除数据
     *
     */
    public void removeAll(List<?> entityList);

    /**
     * 删除运行时数据
     *
     * @param id
     */
    public void removeRunTaskInfo(String id);

    /**
     * 根据用户ID获取部门id
     *
     * @param userId
     *            用户id
     * @return orgId
     */
    public String getOrgIdByUserId(String userId);

    /**
     * 根据用户ID获取部门对象
     *
     * @param userId
     *            用户id
     * @return Org
     */
    public IOrganization getOrgByUserId(String userId);

    /**
     * 根据用户ID获取用户名称
     *
     * @param userId
     *            用户id
     * @return name
     */
    public String getNameById(String userId);

    /**
     * 根据用户ID获取用户对象
     *
     * @param userId
     *            用户id
     * @return User
     */
    public IUser getUserById(String userId);
}
