package com.cesgroup.bdc.wfself.engine.service.impl;

import com.cesgroup.bdc.wfself.def.CommonNode;
import com.cesgroup.bdc.wfself.def.CommonProcDef;
import com.cesgroup.bdc.wfself.def.ITask;
import com.cesgroup.bdc.wfself.engine.constant.ProcConst;
import com.cesgroup.bdc.wfself.engine.service.ITaskService;
import com.cesgroup.bdc.wfself.engine.service.WorkflowDao;
import com.cesgroup.bdc.wfself.entity.ProcInstHisInfo;
import com.cesgroup.bdc.wfself.entity.TaskRunInfo;
import com.cesgroup.bdc.wfself.entity.TaskRunPartic;
import com.cesgroup.bdc.wfself.node.task.UserTask;

import java.util.List;
import java.util.Map;

/**
 * 流程任务service
 * 
 * @author khaoth
 *
 */
public class TaskService extends ProcHandleService implements ITaskService {

    @Override
    public void sign(String taskId, String userId) throws Exception {
        WorkflowDao workflowDao = getWorkflowDao();
        // 查询当前运行任务数据
        TaskRunInfo taskRunInfo = workflowDao.queryTaskRunInfo(taskId);
        // 查询流历史流程数据
        ProcInstHisInfo procInstHisInfo = workflowDao.queryProcInstHisInfo(taskId);
        if (taskRunInfo != null && procInstHisInfo != null) {

            String assgineeId = taskRunInfo.getAssgineeId();
            // 此条件下说明是抢占式任务，需要设置委托人，并删除此节点参与者表中的数据
            if (assgineeId == null || "".equals(assgineeId)) {
                taskRunInfo.setAssgineeId(userId);
                procInstHisInfo.setAssgineeId(userId);
                // 删除参与人数据
                workflowDao.removeAll(
                    workflowDao.queryTaskRunParticList(taskRunInfo.getProcInstId(), taskRunInfo.getTaskCode()));
            }
            taskRunInfo.setStatus(ProcConst.TASK_STATUS_SIGNED);
            procInstHisInfo.setStatus(ProcConst.TASK_STATUS_SIGNED);
            workflowDao.update(taskRunInfo);
            workflowDao.update(procInstHisInfo);
        } else {
            throw new Exception("运行任务或历史数据不存在！id or execId：" + taskId);
        }
    }

    @Override
    public void complete(String taskId, String userId) throws Exception {
        complete(taskId, userId, null);
    }

    @Override
    public void complete(String taskId, String userId, Map<String, String> variableMap) throws Exception {
        WorkflowDao workflowDao = getWorkflowDao();
        // 获取历史流程数据，在运行表中应该存在此taskId的数据。
        ProcInstHisInfo procInstHisInfo = workflowDao.queryProcInstHisInfo(taskId);
        // 流程定义id
        String procDefId = procInstHisInfo.getProcDefId();
        // 流程实例id
        String procInstId = procInstHisInfo.getProcInstId();
        // 人工任务流程节点（一定是人工任务节点code）
        String currUserTaskCode = procInstHisInfo.getNodeCode();
        // 通过流程定义id获取流程定义对象
        CommonProcDef commonProcDef = getProcDef(procDefId);
        // 通过流程定义对象或流程定义map
        Map<String, Object> procDefMap = commonProcDef.getProcDefMap();
        // 通过流程定义对象获取指定节点对象 只可能时任务，如果获取到其他说明流程有问题
        UserTask currUserTask = (UserTask)commonProcDef.getProcDefNode(currUserTaskCode);

        // 当前任务是串行任务
        if (ITask.JOINT_SIGN_SERIAL.equals(currUserTask.getJointSign())) {

            // 判断指定节点是否存在参与人，存在参数人，说明需要串行任务节点
            List<TaskRunPartic> taskRunParticList = workflowDao.queryTaskRunParticList(procInstId, currUserTaskCode);
            if (taskRunParticList != null && !taskRunParticList.isEmpty()) {
                /*
                 * 完成任务，并创建相同的串行任务，此处参数为 null。参数已在第一轮创建时设置入参数表，此处不能设置。
                 */
                this.doHandle(taskId, currUserTask, userId, null);
                return;
            }

            // 当前任务是并行任务
        } else if (ITask.JOINT_SIGN_PARALLEL.equals(currUserTask.getJointSign())) {

            // 有多个任务进入并行网关（聚合），锁住
            synchronized (TaskService.class) {

                // 判断运行表中，指定节点是否都已完成任务（不包过当前任务）。
                List<TaskRunInfo> taskRunInfoList = workflowDao.queryTaskRunInfoList(procInstId, currUserTaskCode);
                if (taskRunInfoList != null && taskRunInfoList.size() > 1) {
                    // 直接完成当前节点，因为运行表中一定存此节点的其他任务，在流转到此节点时，已创建，无需再创建任务。
                    this.completeCurrTask(taskId, userId);
                    return;
                }
            }
        }

        // 这边获取的下一个任务节点只可能时CommonNode类型，不可能时其他类型否则说明存在问题
        CommonNode node = (CommonNode)procDefMap.get(currUserTaskCode);
        // 完成当前任务，流转至下一个节点的执行此处
        this.doHandle(taskId, node, userId, variableMap);
    }

    @Override
    public void rollback(String taskId, String userId) throws Exception {

    }

}
