package com.cesgroup.kingkong.autoconfigure.cis;

import cis.enums.EEncSuit;
import com.sdt.cipherApi.tool.PkiCipher.CipherAPI;
import com.sdt.cipherApi.tool.PkiCipher.CipherParams;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;

/**
 * @author: YYB
 * @description: 加密存储配置
 * @date: 2024/2/5.
 * @modified by:
 */
@Configuration
@ComponentScan(basePackages = { "cis.enums"})
public class CipherConfiguration {

    /**
     * 配置唯一编号不可大于五个字符
     */
    @Value("${cis.cipherId}")
    private String cipherId;

    /**
     * mm机应用名
     */
    @Value("${cis.devAppId}")
    private String devAppId;

    /**
     * mm机类型 参数：DEVICE_FJ DEVICE_311
     */
    @Value("${cis.devType}")
    private String devType;

    /**
     * mm机IP
     */
    @Value("${cis.devIp}")
    private String devIp;

    /**
     * mm机端口
     */
    @Value("${cis.devPort}")
    private String devPort;

    /**
     * 签名私钥名
     */
    @Value("${cis.keyNameSsk}")
    private String keyNameSsk;

    /**
     * 本地保护私钥名
     */
    @Value("${cis.keyNameLpk}")
    private String keyNameLpk;

    /**
     * 加密私钥名
     */
    @Value("${cis.keyNameEsk}")
    private String keyNameEsk;

    /**
     * mm方案 合法值： gw_311 gw_fj
     */
    private  EEncSuit encSuit;

    /**
     * 签名证书名
     */
    @Value("${cis.certName_sign}")
    private String certName_sign;

    /**
     * 加密证书名
     */
    @Value("${cis.certName_enc}")
    private String certName_enc;

    /**
     * 算法类型 参数：10（电子政务） 5（内网认证）
     */
    @Value("${cis.algType}")
    private String algType;

    @Bean
    public CipherAPI cipherAPI(){
        CipherAPI cipherAPI = CipherAPI.getInstance();
        CipherParams cipherParams = new CipherParams(cipherId,devAppId,devType,devIp,devPort,keyNameSsk,keyNameLpk,keyNameEsk, EEncSuit.gw_311_fj, certName_sign, certName_enc, algType);
        try {
            cipherAPI.gwSetCisAppConf(cipherParams,algType);
            cipherAPI.setParams(algType, cipherParams);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return cipherAPI;
    }

}
