package com.cesgroup.kingkong.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.cesgroup.bdc.util.Constant;
import com.cesgroup.bdc.util.CuiPage;
import com.cesgroup.bdc.util.PageVo;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.model.IUser;
import com.cesgroup.kingkong.core.security.service.AuthsystemService;
import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpSession;
import java.util.List;

public abstract class BaseServiceController<T, Service extends IService<T>> {
  @Autowired
  protected Service service;

  @Autowired
  private AuthsystemService authsystemService;

  public Service getBaseService() {
    return service;
  }

  @RequestMapping("/list")
  @ResponseBody
  public List<T> list(T entity) {
    return service.list(Wrappers.<T>query(entity));
  }

  @RequestMapping("/query")
  @ResponseBody
  public PageVo query(CuiPage<T> pager, T entity) {
    //IPage<T> page = new CuiPage<T>();
    IPage<T> p = service.page(pager, Wrappers.<T>query(entity));
    PageVo<T> pv = PageVo.<T>convertFromMybatisplusPage(p);
    return pv;
  }

  @GetMapping("/{id}")
  @ResponseBody
  public T get(@PathVariable String id) {
    return service.getById(id);
  }

  @PostMapping("/save")
  @ResponseBody
  public Boolean save(@RequestBody T user) {
    return service.save(user);
  }

  @PostMapping("/update")
  @ResponseBody
  public Boolean update(@RequestBody T user) {
    return service.updateById(user);
  }

  @DeleteMapping("/{id}")
  @ResponseBody
  public Boolean delete(@PathVariable String id) {
    return service.removeById(id);
  }

  /**
   * 当前登录用户
   *
   * @return 用户信息
   * @author: shen.shaohua
   * @since: 2019/7/3 15:00
   */
  protected IUser getUser() {
    IUser currentUser = (IUser) SecurityUtils.getSubject().getPrincipal();
    return currentUser;
  }

  /**
   * 当前登录用户的部门
   *
   * @return 部门信息
   * @author: shen.shaohua
   * @since: 2019/9/5 10:29
   */
  protected IOrganization getOrg(HttpSession session) {
    IOrganization org = (IOrganization) session.getAttribute(Constant.CURRENT_USER_ORG);
    if (org == null) {
      org = authsystemService.getOrganizationByUserId(getUser().getId());
      session.setAttribute(Constant.CURRENT_USER_ORG, org);
    }
    return org;
  }
}
