(function() {
	function require(path, parent, orig) {
		var resolved = require.resolve(path);
		if (null == resolved) {
			orig = orig || path;
			parent = parent || "root";
			var err = new Error('Failed to require "' + orig + '" from "' + parent + '"');
			err.path = orig;
			err.parent = parent;
			err.require = true;
			throw err
		}
		var module = require.modules[resolved];
		if (!module._resolving && !module.exports) {
			var mod = {};
			mod.exports = {};
			mod.client = mod.component = true;
			module._resolving = true;
			module.call(this, mod.exports, require.relative(resolved), mod);
			delete module._resolving;
			module.exports = mod.exports
		}
		return module.exports
	}
	require.modules = {};
	require.aliases = {};
	require.resolve = function(path) {
		if (path.charAt(0) === "/") path = path.slice(1);
		var paths = [path, path + ".js", path + ".json", path + "/index.js", path + "/index.json"];
		for (var i = 0; i < paths.length; i++) {
			var path = paths[i];
			if (require.modules.hasOwnProperty(path)) return path;
			if (require.aliases.hasOwnProperty(path)) return require.aliases[path]
		}
	};
	require.normalize = function(curr, path) {
		var segs = [];
		if ("." != path.charAt(0)) return path;
		curr = curr.split("/");
		path = path.split("/");
		for (var i = 0; i < path.length; ++i) {
			if (".." == path[i]) {
				curr.pop()
			} else if ("." != path[i] && "" != path[i]) {
				segs.push(path[i])
			}
		}
		return curr.concat(segs).join("/")
	};
	require.register = function(path, definition) {
		require.modules[path] = definition
	};
	require.alias = function(from, to) {
		if (!require.modules.hasOwnProperty(from)) {
			throw new Error('Failed to alias "' + from + '", it does not exist')
		}
		require.aliases[to] = from
	};
	require.relative = function(parent) {
		var p = require.normalize(parent, "..");

		function lastIndexOf(arr, obj) {
			var i = arr.length;
			while (i--) {
				if (arr[i] === obj) return i
			}
			return -1
		}
		function localRequire(path) {
			var resolved = localRequire.resolve(path);
			return require(resolved, parent, path)
		}
		localRequire.resolve = function(path) {
			var c = path.charAt(0);
			if ("/" == c) return path.slice(1);
			if ("." == c) return require.normalize(p, path);
			var segs = parent.split("/");
			var i = lastIndexOf(segs, "deps") + 1;
			if (!i) i = 0;
			path = segs.slice(0, i + 1).join("/") + "/deps/" + path;
			return path
		};
		localRequire.exists = function(path) {
			return require.modules.hasOwnProperty(localRequire.resolve(path))
		};
		return localRequire
	};
	require.register("abpetkov-transitionize/transitionize.js", function(exports, require, module) {
		module.exports = Transitionize;

		function Transitionize(element, props) {
			if (!(this instanceof Transitionize)) return new Transitionize(element, props);
			this.element = element;
			this.props = props || {};
			this.init()
		}
		Transitionize.prototype.isSafari = function() {
			return /Safari/.test(navigator.userAgent) && /Apple Computer/.test(navigator.vendor)
		};
		Transitionize.prototype.init = function() {
			var transitions = [];
			for (var key in this.props) {
				transitions.push(key + " " + this.props[key])
			}
			this.element.style.transition = transitions.join(", ");
			if (this.isSafari()) this.element.style.webkitTransition = transitions.join(", ")
		}
	});
	require.register("switchery/switchery.js", function(exports, require, module) {
		var transitionize = require("transitionize");
		module.exports = Switchery;
		var defaults = {
			//color: "#64bd63",
			//secondaryColor: "#dfdfdf",
			//background: "url(../images/icon.png) no-repeat -19px -562px",
			className: "switchery",
			disabled: false,
			disabledOpacity: .5,
			speed: "0.4s"
		};

		function Switchery(element, options) {
			if (!(this instanceof Switchery)) return new Switchery(element, options);
			this.element = element;
			this.options = options || {};
			for (var i in defaults) {
				if (this.options[i] == null) {
					this.options[i] = defaults[i]
				}
			}
			//ie8下报错
			try{
			if (this.element.type == "checkbox") this.init();
			}catch(e){				
			}
		}
		Switchery.prototype.hide = function() {
			this.element.style.display = "none"
		};
		Switchery.prototype.show = function() {
			var switcher = this.create();
			this.insertAfter(this.element, switcher)
		};
		Switchery.prototype.create = function() {
			this.switcher = document.createElement("span");
			this.jack = document.createElement("small");
			this.switcher.appendChild(this.jack);
			this.switcher.className = this.options.className;
			return this.switcher
		};
		Switchery.prototype.insertAfter = function(reference, target) {
			reference.parentNode.insertBefore(target, reference.nextSibling)
		};
		Switchery.prototype.isChecked = function() {
			return this.element.checked
		};
		Switchery.prototype.isDisabled = function() {
			return this.options.disabled || this.element.disabled
		};
		Switchery.prototype.setPosition = function(clicked) {
			var checked = this.isChecked(),
				switcher = this.switcher,
				jack = this.jack;
			if (clicked && checked) checked = false;
			else if (clicked && !checked) checked = true;
			if (checked === true) {
				this.element.checked = true;
				if (window.getComputedStyle) jack.style.left = parseInt(window.getComputedStyle(switcher).width) - jack.offsetWidth + "px";
				else jack.style.left = parseInt(switcher.currentStyle["width"]) - jack.offsetWidth + "px";
				if (this.options.color) this.colorize();
				this.setSpeed()
			} else {
				jack.style.left = 0;
				this.element.checked = false;
				//this.switcher.style.boxShadow = "inset 0 0 0 0 " + this.options.secondaryColor;
				//this.switcher.style.borderColor = this.options.secondaryColor;
				this.switcher.style.backgroundColor = "";
				this.setSpeed()
			}
		};
		Switchery.prototype.setSpeed = function() {
			var switcherProp = {},
				jackProp = {
					left: this.options.speed.replace(/[a-z]/, "") / 2 + "s"
				};
			if (this.isChecked()) {
				var thisId = this.element.id;
				$('#'+thisId+'').next('.switchery').addClass('switchery-blue');				
				switcherProp = {
					//border: this.options.speed,
					//"box-shadow": this.options.speed,
					//"background-color": this.options.speed.replace(/[a-z]/, "") * 3 + "s",
					//"background": 'url(../css/images/swith.png) no-repeat -19px -562px'
				}
			} else {
				var thisId = this.element.id;
				$('#'+thisId+'').next('.switchery').removeClass('switchery-blue');
				switcherProp = {
					//border: this.options.speed,
					//"box-shadow": this.options.speed
				}
			}
			transitionize(this.switcher, switcherProp);
			transitionize(this.jack, jackProp)
		};
		Switchery.prototype.setAttributes = function() {
			var id = this.element.getAttribute("id"),
				name = this.element.getAttribute("name");
			if (id) this.switcher.setAttribute("id", id);
			if (name) this.switcher.setAttribute("name", name)
		};
		Switchery.prototype.colorize = function() {
			this.switcher.style.backgroundColor = this.options.color;
			this.switcher.style.borderColor = this.options.color;
			this.switcher.style.boxShadow = "inset 0 0 0 16px " + this.options.color
		};
		Switchery.prototype.handleOnchange = function(state) {
			/*
			 * 以下语句在ie8下报错，注释掉

			var evt = new Event("click");
			this.element.dispatchEvent(evt);
			if (typeof Event === "function") {
				var event = new Event("change", {
					cancelable: true
				});
				this.element.dispatchEvent(event)
			} else {
				this.element.fireEvent("onchange")
			}
			*/
		};
		Switchery.prototype.handleClick = function() {
			var self = this,
				switcher = this.switcher;
			if (this.isDisabled() === false) {
				if (switcher.addEventListener) {
					switcher.addEventListener("click", function() {
						self.setPosition(true);
						self.handleOnchange(self.element.checked)
					})
				} else {
					switcher.attachEvent("onclick", function() {
						self.setPosition(true);
						self.handleOnchange(self.element.checked)
					})
				}
			} else {
				this.element.disabled = true;
				this.switcher.style.opacity = this.options.disabledOpacity
			}
		};
		Switchery.prototype.init = function() {
			this.hide();
			this.show();
			this.setPosition();
			this.setAttributes();
			this.handleClick()
		}
	});
	require.alias("abpetkov-transitionize/transitionize.js", "switchery/deps/transitionize/transitionize.js");
	require.alias("abpetkov-transitionize/transitionize.js", "switchery/deps/transitionize/index.js");
	require.alias("abpetkov-transitionize/transitionize.js", "transitionize/index.js");
	require.alias("abpetkov-transitionize/transitionize.js", "abpetkov-transitionize/index.js");
	require.alias("switchery/switchery.js", "switchery/index.js");
	if (typeof exports == "object") {
		module.exports = require("switchery")
	} else if (typeof define == "function" && define.amd) {
		define(function() {
			return require("switchery")
		})
	} else {
		this["Switchery"] = require("switchery")
	}
})();
/*
// Default
  // if-else statement used only for fixing <IE9 issues
  if (Array.prototype.forEach) {
	var elems = Array.prototype.slice.call(document.querySelectorAll('.js-switch'));

	elems.forEach(function(html) {
	  var switchery = new Switchery(html);
	});
  } else {
	var elems = document.querySelectorAll('.js-switch');

	for (var i = 0; i < elems.length; i++) {
	  var switchery = new Switchery(elems[i]);
	}
  }

  // Disabled switch
  var disabled = document.querySelector('.js-switch-disabled');
  var switchery = new Switchery(disabled, { disabled: true });

  var disabledOpacity = document.querySelector('.js-switch-disabled-opacity');
  var switchery = new Switchery(disabledOpacity, { disabled: true, disabledOpacity: 0.75 });

  // Colored switches
  var blue = document.querySelector('.js-switch-blue');
  var switchery = new Switchery(blue, { color: '#41b7f1' });

  var pink = document.querySelector('.js-switch-pink');
  var switchery = new Switchery(pink, { color: '#ff7791' });

  var teal = document.querySelector('.js-switch-teal');
  var switchery = new Switchery(teal, { color: '#3cc8ad' });

  var red = document.querySelector('.js-switch-red');
  var switchery = new Switchery(red, { color: '#db5554' });

  var secondary = document.querySelector('.js-switch-secondary');
  var switchery = new Switchery(secondary, { color: '#fec200', secondaryColor: '#ff8787' });
*/